@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2006 Chris Vine
+/* Copyright (C) 2001 to 2006 and 2009 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -9,39 +9,18 @@ COPYING distributed with the source files.
 
 #include <fstream>
 #include <cstdlib>
+#include <ios>
+#include <ostream>
 
-#include <gtk/gtktable.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtkmisc.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkbbox.h>
-#include <gtk/gtkhbbox.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtkcontainer.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkiconfactory.h>
-#include <gtk/gtktooltips.h>
-#include <gtk/gtkliststore.h>
-#include <gtk/gtkcellrenderer.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtktreeselection.h>
-#include <gtk/gtktreeviewcolumn.h>
-
-#include <gdk/gdkpixbuf.h>
+#include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h> // the key codes are here
 
 #include "addressbook.h"
 #include "dialogs.h"
-#include "utils/utf8_utils.h"
 #include "utils/tree_path_handle.h"
 #include "addressbook_icons.h"
 
-#ifdef HAVE_OSTREAM
-#include <ios>
-#include <ostream>
-#endif
+#include <c++-gtk-utils/convert.h>
 
 #ifdef ENABLE_NLS
 #include <libintl.h>
@@ -118,7 +97,7 @@ AddressBook::AddressBook(const int size, GtkWindow* parent_p):
   GtkScrolledWindow* scrolled_window_p = GTK_SCROLLED_WINDOW(gtk_scrolled_window_new(0, 0));
 
   gtk_button_box_set_layout(GTK_BUTTON_BOX(button_box_p), GTK_BUTTONBOX_END);
-  gtk_button_box_set_spacing(GTK_BUTTON_BOX(button_box_p), standard_size/2);
+  gtk_box_set_spacing(GTK_BOX(button_box_p), standard_size/2);
 
   gtk_container_add(GTK_CONTAINER(button_box_p), cancel_button_p);
   gtk_container_add(GTK_CONTAINER(button_box_p), ok_button_p);
@@ -217,11 +196,10 @@ AddressBook::AddressBook(const int size, GtkWindow* parent_p):
 		   GtkAttachOptions(GTK_FILL | GTK_EXPAND), GtkAttachOptions(GTK_FILL | GTK_EXPAND),
 		   0, 0);
 
-  GtkTooltips* tooltips_p = gtk_tooltips_new();
-  gtk_tooltips_set_tip(tooltips_p, add_button_p, gettext("Add new address"), "");
-  gtk_tooltips_set_tip(tooltips_p, delete_button_p, gettext("Delete address"), "");
-  gtk_tooltips_set_tip(tooltips_p, up_button_p, gettext("Move address up"), "");
-  gtk_tooltips_set_tip(tooltips_p, down_button_p, gettext("Move address down"), "");
+  gtk_widget_set_tooltip_text(add_button_p, gettext("Add new address"));
+  gtk_widget_set_tooltip_text(delete_button_p, gettext("Delete address"));
+  gtk_widget_set_tooltip_text(up_button_p, gettext("Move address up"));
+  gtk_widget_set_tooltip_text(down_button_p, gettext("Move address down"));
 
   gtk_table_attach(table_p, GTK_WIDGET(list_table_p),
 		   0, 1, 0, 1,
@@ -246,13 +224,21 @@ AddressBook::AddressBook(const int size, GtkWindow* parent_p):
   g_signal_connect(G_OBJECT(down_button_p), "clicked",
 		   G_CALLBACK(AddressBookCB::addr_book_button_clicked), this);
   
+#if GTK_CHECK_VERSION(2,20,0)
+  gtk_widget_set_can_default(ok_button_p, true);
+  gtk_widget_set_can_default(cancel_button_p, true);
+  gtk_widget_set_can_default(add_button_p, false);
+  gtk_widget_set_can_default(delete_button_p, false);
+  gtk_widget_set_can_focus(up_button_p, false);
+  gtk_widget_set_can_focus(down_button_p, false);
+#else
   GTK_WIDGET_SET_FLAGS(ok_button_p, GTK_CAN_DEFAULT);
   GTK_WIDGET_SET_FLAGS(cancel_button_p, GTK_CAN_DEFAULT);
-
   GTK_WIDGET_UNSET_FLAGS(add_button_p, GTK_CAN_DEFAULT);
   GTK_WIDGET_UNSET_FLAGS(delete_button_p, GTK_CAN_DEFAULT);
   GTK_WIDGET_UNSET_FLAGS(up_button_p, GTK_CAN_FOCUS);
   GTK_WIDGET_UNSET_FLAGS(down_button_p, GTK_CAN_FOCUS);
+#endif
 
   gtk_container_set_border_width(GTK_CONTAINER(table_p), standard_size/3);
   gtk_container_add(GTK_CONTAINER(get_win()), GTK_WIDGET(table_p));
@@ -304,7 +290,7 @@ GcharSharedHandle AddressBook::get_number(void) {
 void AddressBook::add_address_prompt(void) {
 
   AddressDialog* dialog_p = new AddressDialog(standard_size, this->get_win());
-  dialog_p->accepted.connect(sigc::mem_fun(*this, &AddressBook::add_address_impl));
+  dialog_p->accepted.connect(Callback::make(*this, &AddressBook::add_address_impl));
   // there is no memory leak -- AddressDailog will delete its own memory
   // when it is closed
 }
@@ -328,7 +314,7 @@ void AddressBook::delete_address_prompt(void) {
     PromptDialog* dialog_p = new PromptDialog(gettext("Delete selected address?"),
 					      gettext("efax-gtk: Delete address"), standard_size,
 					      get_win());
-    dialog_p->accepted.connect(sigc::mem_fun(*this, &AddressBook::delete_address_impl));
+    dialog_p->accepted.connect(Callback::make(*this, &AddressBook::delete_address_impl));
     // there is no memory leak -- the memory will be deleted when PromptDialog closes
   }
   else beep();
@@ -563,7 +549,11 @@ void AddressDialogCB::addr_dialog_selected(GtkWidget* widget_p, void* data) {
 gboolean AddressDialogCB::addr_dialog_key_press_event(GtkWidget*, GdkEventKey* event_p, void* data) {
   AddressDialog* instance_p = static_cast<AddressDialog*>(data);
   int keycode = event_p->keyval;
+#if GTK_CHECK_VERSION(2,20,0)
+  bool cancel_focus = gtk_widget_has_focus(instance_p->cancel_button_p);
+#else
   bool cancel_focus = GTK_WIDGET_HAS_FOCUS(instance_p->cancel_button_p);
+#endif
 
   if (keycode == GDK_Return && !cancel_focus) {
     if (instance_p->selected_impl()) instance_p->close();
@@ -600,7 +590,7 @@ AddressDialog::AddressDialog(const int standard_size, GtkWindow* parent_p):
   GtkTable* table_p = GTK_TABLE(gtk_table_new(3, 2, false));
 
   gtk_button_box_set_layout(GTK_BUTTON_BOX(button_box_p), GTK_BUTTONBOX_END);
-  gtk_button_box_set_spacing(GTK_BUTTON_BOX(button_box_p), standard_size/2);
+  gtk_box_set_spacing(GTK_BOX(button_box_p), standard_size/2);
 
   gtk_container_add(GTK_CONTAINER(button_box_p), cancel_button_p);
   gtk_container_add(GTK_CONTAINER(button_box_p), ok_button_p);
@@ -635,8 +625,13 @@ AddressDialog::AddressDialog(const int standard_size, GtkWindow* parent_p):
   g_signal_connect(G_OBJECT(get_win()), "key_press_event",
 		   G_CALLBACK(AddressDialogCB::addr_dialog_key_press_event), this);
 
+#if GTK_CHECK_VERSION(2,20,0)
+  gtk_widget_set_can_default(ok_button_p, true);
+  gtk_widget_set_can_default(cancel_button_p, true);
+#else
   GTK_WIDGET_SET_FLAGS(ok_button_p, GTK_CAN_DEFAULT);
   GTK_WIDGET_SET_FLAGS(cancel_button_p, GTK_CAN_DEFAULT);
+#endif
 
   gtk_container_add(GTK_CONTAINER(get_win()), GTK_WIDGET(table_p));
   
