@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2006 Chris Vine
+/* Copyright (C) 2001 to 2010 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -13,49 +13,18 @@ COPYING distributed with the source files.
 #include <cstdlib>
 #include <cstring>
 
-#include <gtk/gtktable.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkbox.h>
-#include <gtk/gtkbbox.h>
-#include <gtk/gtkhbbox.h>
-#include <gtk/gtktextbuffer.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtkcontainer.h>
-#include <gtk/gtkversion.h>
-#include <gtk/gtkstock.h>
-
-#include <gdk/gdkpixbuf.h>
 #include <gdk/gdkkeysyms.h> // the key codes are here
 
-#if GTK_CHECK_VERSION(2,4,0)
-#  undef EFAX_GTK_USE_FILE_SEL
-#else
-#  define EFAX_GTK_USE_FILE_SEL 1
-#endif
-
-#ifdef EFAX_GTK_USE_FILE_SEL
-#  include <gtk/gtkfilesel.h>
-#  define EFAX_GTK_FILE_DIALOG_NEW gtk_file_selection_new(0)
-#else
-#  include <gtk/gtkfilechooserdialog.h>
-#  include <glib/glist.h>
-#  define EFAX_GTK_FILE_DIALOG_NEW (gtk_file_chooser_dialog_new(0, 0, \
-                                   GTK_FILE_CHOOSER_ACTION_OPEN, \
-                                   gettext("View"), GTK_RESPONSE_NONE, \
-                                   GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, \
-                                   GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, \
-                                   static_cast<void*>(0)))
-#endif
-
 #include <pango/pango-font.h>
 #include <pango/pango-layout.h>
 
 #include "dialogs.h"
-#include "utils/shared_handle.h"
-#include "utils/gobj_handle.h"
-#include "utils/mutex.h"
-#include "utils/utf8_utils.h"
+
+#include <c++-gtk-utils/gobj_handle.h>
+#include <c++-gtk-utils/shared_handle.h>
+#include <c++-gtk-utils/mutex.h>
+#include <c++-gtk-utils/convert.h>
+
 #include "gpl.h"
 
 #ifdef ENABLE_NLS
@@ -84,40 +53,19 @@ void FileReadSelectDialogCB::file_view_file(GtkWidget*, void* data) {
 } // anonymous namespace
 
 FileReadSelectDialog::FileReadSelectDialog(int size, bool multi_files, GtkWindow* parent_p):
-                                WinBase(gettext("efax-gtk: File to fax"),
-					prog_config.window_icon_h,
-					true, parent_p,
-					GTK_WINDOW(EFAX_GTK_FILE_DIALOG_NEW)),
-				standard_size(size) {
+            WinBase(gettext("efax-gtk: File to fax"),
+		    prog_config.window_icon_h,
+		    true, parent_p,
+		    GTK_WINDOW(gtk_file_chooser_dialog_new(0, 0,
+					GTK_FILE_CHOOSER_ACTION_OPEN,
+					gettext("View"), GTK_RESPONSE_NONE,
+					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
+							   static_cast<void*>(0)))),
+	    standard_size(size) {
 
   GtkWidget* view_button_p;
 
-#ifdef EFAX_GTK_USE_FILE_SEL
-  GtkFileSelection* file_object_p = GTK_FILE_SELECTION(get_win());
-
-  gtk_file_selection_set_select_multiple(file_object_p, multi_files);
-  if (!prog_config.working_dir.empty()) {
-    std::string temp(prog_config.working_dir);
-    temp +=  "/faxout/";
-    gtk_file_selection_set_filename(file_object_p, temp.c_str());
-  }
-
-  gtk_file_selection_hide_fileop_buttons(file_object_p);
-
-  GtkBox* hbox_p = GTK_BOX(file_object_p->action_area);
-  gtk_box_set_homogeneous(hbox_p, false);
-
-  GtkWidget* label_p = gtk_label_new(0);
-  view_button_p = gtk_button_new_with_label(gettext("View"));
-  gtk_box_pack_start(hbox_p, label_p, true, false, 0);
-  gtk_box_pack_start(hbox_p, view_button_p, false, false, 0);
-
-  GTK_WIDGET_SET_FLAGS(view_button_p, GTK_CAN_DEFAULT);
-
-  ok_button_p = file_object_p->ok_button;
-  cancel_button_p = file_object_p->cancel_button;
-
-#else
   GtkFileChooser* file_object_p = GTK_FILE_CHOOSER(get_win());
 
   gtk_file_chooser_set_select_multiple(file_object_p, multi_files);
@@ -127,7 +75,11 @@ FileReadSelectDialog::FileReadSelectDialog(int size, bool multi_files, GtkWindow
     gtk_file_chooser_set_current_folder(file_object_p, temp.c_str());
   }
 
+#if GTK_CHECK_VERSION(2,14,0)
+  GtkBox* hbox_p = GTK_BOX(gtk_dialog_get_action_area(GTK_DIALOG(get_win())));
+#else
   GtkBox* hbox_p = GTK_BOX(GTK_DIALOG(get_win())->action_area);
+#endif
   gtk_box_set_homogeneous(hbox_p, true);
 
   GList* button_list = gtk_container_get_children(GTK_CONTAINER(hbox_p));
@@ -144,8 +96,6 @@ FileReadSelectDialog::FileReadSelectDialog(int size, bool multi_files, GtkWindow
 
   g_list_free(button_list);
 
-#endif
-
   g_signal_connect(G_OBJECT(ok_button_p), "clicked",
 		   G_CALLBACK(FileReadSelectDialogCB::file_selected), this);
   g_signal_connect(G_OBJECT(cancel_button_p), "clicked",
@@ -157,13 +107,6 @@ FileReadSelectDialog::FileReadSelectDialog(int size, bool multi_files, GtkWindow
   gtk_window_set_position(get_win(), GTK_WIN_POS_CENTER_ON_PARENT);
 
   gtk_widget_show_all(GTK_WIDGET(get_win()));
-
-#ifdef EFAX_GTK_USE_FILE_SEL
-  // now get and set the button size for view_button
-  gtk_widget_set_size_request(view_button_p,
-			      cancel_button_p->allocation.width,
-			      cancel_button_p->allocation.height);
-#endif
 }
 
 void FileReadSelectDialog::on_delete_event(void) {
@@ -291,26 +234,14 @@ void FileReadSelectDialog::delete_parms(std::pair<const char*, char* const*> par
 }
 
 std::string FileReadSelectDialog::get_filename_string(void) {
-#ifdef EFAX_GTK_USE_FILE_SEL
-  return std::string((const char*)gtk_file_selection_get_filename(GTK_FILE_SELECTION(get_win())));
-#else
   GcharScopedHandle file_name_h(gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(get_win())));
   if (file_name_h.get()) return std::string((const char*)file_name_h.get());
   return std::string();
-#endif
 }
 
 void FileReadSelectDialog::set_result(void) {
 
   result.clear();
-#ifdef EFAX_GTK_USE_FILE_SEL
-  gchar** file_array = gtk_file_selection_get_selections(GTK_FILE_SELECTION(get_win()));
-  gchar** temp_p = file_array;
-  for (; *temp_p; temp_p++) {
-    result.push_back((const char*)*temp_p);
-  }
-  g_strfreev (file_array);
-#else
   GSList* file_list_p = gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(get_win()));
   void* elem = g_slist_nth_data(file_list_p, 0); // first element
   for (guint count = 1; elem; count++) {
@@ -319,7 +250,7 @@ void FileReadSelectDialog::set_result(void) {
     elem = g_slist_nth_data(file_list_p, count);
   }
   g_slist_free(file_list_p);
-#endif
+
   try {
     std::transform(result.begin(), result.end(), result.begin(),
 		   Utf8::filename_to_utf8);
@@ -459,9 +390,15 @@ GplDialog::GplDialog(int size): WinBase(gettext("efax-gtk: Conditions, Notices a
   
   gtk_widget_grab_focus(GTK_WIDGET(get_win()));
 
+#if GTK_CHECK_VERSION(2,20,0)
+  gtk_widget_set_can_focus(GTK_WIDGET(text_view_p), false);
+  gtk_widget_set_can_focus(accept_button_p, false);
+  gtk_widget_set_can_focus(reject_button_p, false);
+#else
   GTK_WIDGET_UNSET_FLAGS(GTK_WIDGET(text_view_p), GTK_CAN_FOCUS);
   GTK_WIDGET_UNSET_FLAGS(accept_button_p, GTK_CAN_FOCUS);
   GTK_WIDGET_UNSET_FLAGS(reject_button_p, GTK_CAN_FOCUS);
+#endif
 
   gtk_widget_show_all(GTK_WIDGET(get_win()));
 }
@@ -518,20 +455,21 @@ InfoDialog::InfoDialog(const char* text, const char* caption,
                        GtkMessageType message_type, GtkWindow* parent_p, bool modal):
                        WinBase(caption, prog_config.window_icon_h,
 			       modal, parent_p,
-			       GTK_WINDOW(gtk_message_dialog_new(parent_p, GtkDialogFlags(0),
+			       GTK_WINDOW(gtk_message_dialog_new(0, GtkDialogFlags(0),
 					  message_type, GTK_BUTTONS_CLOSE, text))) {
 
   // make further specialisations for a message dialog object
 
-  if (!GTK_CHECK_VERSION(2, 4, 0)) {
-    gtk_dialog_set_has_separator(GTK_DIALOG(get_win()), false);
-  }
+#if GTK_CHECK_VERSION(2,14,0)
+  GtkWidget* action_area_p = gtk_dialog_get_action_area(GTK_DIALOG(get_win()));
+#else
+  GtkWidget* action_area_p = GTK_DIALOG(get_win())->action_area;
+#endif
+  gtk_button_box_set_layout(GTK_BUTTON_BOX(action_area_p),
+			    GTK_BUTTONBOX_SPREAD);
 
   gtk_window_set_type_hint(get_win(), GDK_WINDOW_TYPE_HINT_DIALOG);
 
-  gtk_button_box_set_layout(GTK_BUTTON_BOX(GTK_DIALOG(get_win())->action_area),
-			    GTK_BUTTONBOX_SPREAD);
-
   g_signal_connect(G_OBJECT(get_win()), "response",
 		   G_CALLBACK(InfoDialogCB::info_selected), this);
 
@@ -541,71 +479,61 @@ InfoDialog::InfoDialog(const char* text, const char* caption,
   gtk_widget_show_all(GTK_WIDGET(get_win()));
 }
 
+void InfoDialog::on_delete_event(void) {
+  // the response event handler will cause close() to be called for
+  // the GTK_RESPONSE_DELETE_EVENT id, so just return when we
+  // subsequently get the delete event
+  return;
+}
 
 namespace { // callbacks for internal use only
 
-void PromptDialogCB::prompt_selected(GtkWidget* widget_p, void* data) {
+void PromptDialogCB::prompt_selected(GtkDialog*, int id, void* data) {
   PromptDialog* instance_p = static_cast<PromptDialog*>(data);
-  gtk_widget_hide_all(GTK_WIDGET(instance_p->get_win()));
-  if (widget_p == instance_p->ok_button_p) {
+  if (id == GTK_RESPONSE_ACCEPT) {
     instance_p->result = true;
     instance_p->accepted();
+    instance_p->close();
   }
-  else if (widget_p == instance_p->cancel_button_p) {
+  else if (id == GTK_RESPONSE_REJECT) {
     instance_p->result = false;
     instance_p->rejected();
+    instance_p->close();
   }
-  else write_error("Callback error in PromptDialogCB::prompt_selected()\n");
-  instance_p->close();
 }
 
 } // anonymous namespace
 
 PromptDialog::PromptDialog(const char* text, const char* caption,
 			   int standard_size, GtkWindow* parent_p, bool modal):
-                               WinBase(caption, prog_config.window_icon_h,
-				       modal, parent_p),
-                               result(false) {
+                           WinBase(caption, prog_config.window_icon_h,
+				   modal, parent_p,
+				   GTK_WINDOW(gtk_dialog_new_with_buttons(0, 0,
+						  GTK_DIALOG_NO_SEPARATOR,
+						  GTK_STOCK_CANCEL,
+						  GTK_RESPONSE_REJECT,
+						  GTK_STOCK_OK,
+						  GTK_RESPONSE_ACCEPT,
+						  static_cast<void*>(0)))),
+			   result(false) {
 
-  ok_button_p = gtk_button_new_from_stock(GTK_STOCK_OK);
-  cancel_button_p = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
-  GtkWidget* button_box_p = gtk_hbutton_box_new();
   GtkWidget* label_p = gtk_label_new(text);
-  GtkTable* table_p = GTK_TABLE(gtk_table_new(2, 1, false));
-
-  gtk_button_box_set_layout(GTK_BUTTON_BOX(button_box_p), GTK_BUTTONBOX_END);
-  gtk_button_box_set_spacing(GTK_BUTTON_BOX(button_box_p), standard_size/2);
-
   gtk_label_set_line_wrap(GTK_LABEL(label_p), true);
+  gtk_misc_set_padding(GTK_MISC(label_p), standard_size/2, standard_size/2);
+#if GTK_CHECK_VERSION(2,14,0)
+  GtkWidget* content_area_p = gtk_dialog_get_content_area(GTK_DIALOG(get_win()));
+#else
+  GtkWidget* content_area_p = GTK_DIALOG(get_win())->vbox;
+#endif
+  gtk_container_add(GTK_CONTAINER(content_area_p), 
+		    label_p);
 
-  gtk_container_add(GTK_CONTAINER(button_box_p), cancel_button_p);
-  gtk_container_add(GTK_CONTAINER(button_box_p), ok_button_p);
+  gtk_dialog_set_default_response(GTK_DIALOG(get_win()), GTK_RESPONSE_ACCEPT);
 
-  gtk_table_attach(table_p, label_p,
-		   0, 1, 0, 1,
-		   GtkAttachOptions(GTK_FILL | GTK_EXPAND), GtkAttachOptions(GTK_FILL | GTK_EXPAND),
-		   standard_size/2, standard_size/4);
-
-  gtk_table_attach(table_p, button_box_p,
-		   0, 1, 1, 2,
-		   GtkAttachOptions(GTK_FILL | GTK_EXPAND), GTK_SHRINK,
-		   standard_size/2, standard_size/4);
-
-  g_signal_connect(G_OBJECT(ok_button_p), "clicked",
-		   G_CALLBACK(PromptDialogCB::prompt_selected), this);
-  g_signal_connect(G_OBJECT(cancel_button_p), "clicked",
-		   G_CALLBACK(PromptDialogCB::prompt_selected), this);
-
-  GTK_WIDGET_SET_FLAGS(ok_button_p, GTK_CAN_DEFAULT);
-  GTK_WIDGET_SET_FLAGS(cancel_button_p, GTK_CAN_DEFAULT);
-
-  gtk_container_add(GTK_CONTAINER(get_win()), GTK_WIDGET(table_p));
-  
   gtk_window_set_type_hint(get_win(), GDK_WINDOW_TYPE_HINT_DIALOG);
 
-  gtk_container_set_border_width(GTK_CONTAINER(get_win()), standard_size/2);
-
-  gtk_widget_grab_focus(ok_button_p);
+  g_signal_connect(G_OBJECT(get_win()), "response",
+		   G_CALLBACK(PromptDialogCB::prompt_selected), this);
 
   gtk_window_set_position(get_win(), GTK_WIN_POS_CENTER_ON_PARENT);
   gtk_window_set_resizable(get_win(), false);
@@ -618,8 +546,100 @@ int PromptDialog::get_exec_val(void) const {
 }
 
 void PromptDialog::on_delete_event(void) {
-  gtk_widget_hide_all(GTK_WIDGET(get_win()));
   result = false;
   rejected();
   close();
 }
+
+namespace { // callbacks for internal use only
+
+void AboutEfaxGtkDialogCB::about_efax_gtk_selected(GtkDialog*, int, void* data) {
+  static_cast<AboutEfaxGtkDialog*>(data)->close();
+}
+
+} // anonymous namespace
+
+AboutEfaxGtkDialog::AboutEfaxGtkDialog(GtkWindow* parent_p, bool modal):
+                                       WinBase(gettext("About efax-gtk"), prog_config.window_icon_h,
+					       modal, parent_p,
+					       GTK_WINDOW(gtk_about_dialog_new())) {
+
+  gtk_window_set_type_hint(get_win(), GDK_WINDOW_TYPE_HINT_DIALOG);
+
+  g_signal_connect(G_OBJECT(get_win()), "response",
+		   G_CALLBACK(AboutEfaxGtkDialogCB::about_efax_gtk_selected), this);
+
+  gtk_window_set_resizable(get_win(), false);
+
+  if (parent_p && modal) gtk_window_set_position(get_win(), GTK_WIN_POS_CENTER_ON_PARENT);
+
+  gtk_about_dialog_set_program_name(GTK_ABOUT_DIALOG(get_win()), "efax-gtk");
+  gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(get_win()), VERSION);
+  gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(get_win()),
+				 gettext("Copyright (C) 2001 - 2010 Chris Vine"));
+  gtk_about_dialog_set_license(GTK_ABOUT_DIALOG(get_win()),
+			       gpl_copyright_msg()->c_str()); // this function calls g_strdup()
+  gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(get_win()),
+				gettext("This program was written by Chris Vine "
+					"and is released under the GNU General "
+					"Public License, version 2. It is a front "
+					"end for efax. efax was written by Ed Casas"));
+
+  gtk_widget_show_all(GTK_WIDGET(get_win()));
+}
+
+void AboutEfaxGtkDialog::on_delete_event(void) {
+  // the response event handler will cause close() to be called for
+  // the GTK_RESPONSE_DELETE_EVENT id, so just return when we
+  // subsequently get the delete event
+  return;
+}
+
+namespace { // callbacks for internal use only
+
+void AboutEfaxDialogCB::about_efax_selected(GtkDialog*, int, void* data) {
+  static_cast<AboutEfaxDialog*>(data)->close();
+}
+
+} // anonymous namespace
+
+AboutEfaxDialog::AboutEfaxDialog(GtkWindow* parent_p, bool modal):
+                                 WinBase(gettext("About efax"), prog_config.window_icon_h,
+					 modal, parent_p,
+					 GTK_WINDOW(gtk_about_dialog_new())) {
+
+#if GTK_CHECK_VERSION(2,14,0)
+  GtkWidget* action_area_p = gtk_dialog_get_action_area(GTK_DIALOG(get_win()));
+#else
+  GtkWidget* action_area_p = GTK_DIALOG(get_win())->action_area;
+#endif
+  gtk_button_box_set_layout(GTK_BUTTON_BOX(action_area_p),
+			    GTK_BUTTONBOX_SPREAD);
+
+  gtk_window_set_type_hint(get_win(), GDK_WINDOW_TYPE_HINT_DIALOG);
+
+  g_signal_connect(G_OBJECT(get_win()), "response",
+		   G_CALLBACK(AboutEfaxDialogCB::about_efax_selected), this);
+
+  gtk_window_set_resizable(get_win(), false);
+
+  if (parent_p && modal) gtk_window_set_position(get_win(), GTK_WIN_POS_CENTER_ON_PARENT);
+
+  gtk_about_dialog_set_program_name(GTK_ABOUT_DIALOG(get_win()), "efax");
+  gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(get_win()),
+				 gettext("efax is copyright (C) 1999 Ed Casas"));
+  gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(get_win()),
+				gettext("This program (efax-gtk) is a front end "
+					"for efax. efax is a program released "
+					"under the GNU General Public License, "
+					"version 2 by Ed Casas"));
+
+  gtk_widget_show_all(GTK_WIDGET(get_win()));
+}
+
+void AboutEfaxDialog::on_delete_event(void) {
+  // the response event handler will cause close() to be called for
+  // the GTK_RESPONSE_DELETE_EVENT id, so just return when we
+  // subsequently get the delete event
+  return;
+}
