@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2005 Chris Vine
+/* Copyright (C) 2001 to 2010 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -15,18 +15,11 @@ COPYING distributed with the source files.
 #include <vector>
 #include <utility>
 
-#include <gtk/gtkwidget.h>
-#include <gtk/gtkwindow.h>
-#include <gtk/gtkdialog.h>
-#include <gtk/gtkmessagedialog.h>
-#include <gtk/gtktextview.h>
-#include <gtk/gtkstyle.h>
+#include <gtk/gtk.h>
+#include <gdk/gdk.h>
 
-#include <gdk/gdkevents.h>
-
-#include <sigc++/sigc++.h>
-
-#include "utils/window.h"
+#include <c++-gtk-utils/window.h>
+#include <c++-gtk-utils/emitter.h>
 
 
 namespace { // we put the functions in anonymous namespace in dialogs.cpp
@@ -95,14 +88,16 @@ public:
   GplDialog(int standard_size);
 };
 
-namespace { // we put the functions in anonymous namespace in dialogs.cpp
-            // so they are not exported at link time
+namespace { // we put the function in anonymous namespace in dialogs.cpp
+            // so it is not exported at link time
 namespace InfoDialogCB {
   extern "C" void info_selected(GtkDialog*, int, void*);
 }
 }   // anonymous namespace
 
 class InfoDialog: public WinBase {
+protected:
+  virtual void on_delete_event(void);
 public:
   friend void InfoDialogCB::info_selected(GtkDialog*, int, void*);
 
@@ -110,29 +105,58 @@ public:
 	     GtkMessageType message_type, GtkWindow* parent_p, bool modal = true);
 };
 
-
-namespace { // we put the functions in anonymous namespace in dialogs.cpp
-            // so they are not exported at link time
+namespace { // we put the function in anonymous namespace in dialogs.cpp
+            // so it is not exported at link time
 namespace PromptDialogCB {
-  extern "C" void prompt_selected(GtkWidget*, void*);
+  extern "C" void prompt_selected(GtkDialog*, int, void*);
 }
 }   // anonymous namespace
 
 class PromptDialog: public WinBase {
   bool result;
-  GtkWidget* ok_button_p;
-  GtkWidget* cancel_button_p;
 protected:
   virtual int get_exec_val(void) const;
   virtual void on_delete_event(void);
 public:
-  friend void PromptDialogCB::prompt_selected(GtkWidget*, void*);
+  friend void PromptDialogCB::prompt_selected(GtkDialog*, int, void*);
 
-  sigc::signal0<void> accepted;
-  sigc::signal0<void> rejected;
+  Emitter accepted;
+  Emitter rejected;
 
   PromptDialog(const char* text, const char* caption,
 	       int standard_size, GtkWindow* parent_p, bool modal = true);
 };
 
+namespace { // we put the function in anonymous namespace in dialogs.cpp
+            // so it is not exported at link time
+namespace AboutEfaxGtkDialogCB {
+  extern "C" void about_efax_gtk_selected(GtkDialog*, int, void*);
+}
+}   // anonymous namespace
+
+class AboutEfaxGtkDialog: public WinBase {
+protected:
+  virtual void on_delete_event(void);
+public:
+  friend void AboutEfaxGtkDialogCB::about_efax_gtk_selected(GtkDialog*, int, void*);
+
+  AboutEfaxGtkDialog(GtkWindow* parent_p = 0, bool modal = false);
+};
+
+namespace { // we put the function in anonymous namespace in dialogs.cpp
+            // so it is not exported at link time
+namespace AboutEfaxDialogCB {
+  extern "C" void about_efax_selected(GtkDialog*, int, void*);
+}
+}   // anonymous namespace
+
+class AboutEfaxDialog: public WinBase {
+protected:
+  virtual void on_delete_event(void);
+public:
+  friend void AboutEfaxDialogCB::about_efax_selected(GtkDialog*, int, void*);
+
+  AboutEfaxDialog(GtkWindow* parent_p = 0, bool modal = false);
+};
+
 #endif
