@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2007 Chris Vine
+/* Copyright (C) 2001 to 2007 and 2009 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -6,18 +6,13 @@ COPYING distributed with the source files.
 
 */
 
-// something in gtkprintjob.h clashes with the C++ headers with
-// libstdc++-5 and it needs to be included here to obtain a
-// clean compile
-#include <gtk/gtkversion.h>
-#if GTK_CHECK_VERSION(2,10,0)
-#include <gtk/gtkprintjob.h>
-#endif
-
 #include <stdlib.h>  // include <stdlib.h> rather than <cstdlib> for mkstemp
 #include <iomanip>
 #include <memory>
 #include <cstring>
+#include <ios>
+#include <ostream>
+#include <sstream>
 
 #include <unistd.h>
 #include <sys/types.h>
@@ -27,61 +22,27 @@ COPYING distributed with the source files.
 #include <errno.h>
 #include <pthread.h>
 
-#include <gtk/gtkcontainer.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkbox.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkbbox.h>
-#include <gtk/gtkhbbox.h>
-#include <gtk/gtkpaned.h>
-#include <gtk/gtkhpaned.h>
-#include <gtk/gtkframe.h>
-#include <gtk/gtkmisc.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtktoolbar.h>
-#include <gtk/gtktooltips.h>
-#include <gtk/gtkiconfactory.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkstock.h>
-
-#include <gdk/gdkcolor.h>
-#include <gdk/gdkpixbuf.h>
+#include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h> // the key codes are here
-
-#include <glib/gtimer.h>
+#include <glib.h>
 
 #include "fax_list.h"
 #include "dialogs.h"
 #include "fax_list_icons.h"
-#include "utils/thread.h"
-#include "utils/utf8_utils.h"
+
 #include "utils/toolbar_append_widget.h"
-#include "utils/callback.h"
+#include "utils/mono_tiff_print_manager.h"
+#include "utils/icon_info_handle.h"
 
-#ifdef HAVE_OSTREAM
-#include <ios>
-#include <ostream>
-#endif
-
-#ifdef HAVE_STRINGSTREAM
-#include <sstream>
-#else
-#include <strstream>
-#endif
+#include <c++-gtk-utils/thread.h>
+#include <c++-gtk-utils/callback.h>
+#include <c++-gtk-utils/gerror_handle.h>
+#include <c++-gtk-utils/convert.h>
 
 #ifdef HAVE_STREAM_IMBUE
 #include <locale>
 #endif
 
-#if GTK_CHECK_VERSION(2,4,0)
-#include <gtk/gtktoolitem.h>
-#include <gtk/gtkseparatortoolitem.h>
-#endif
-
 #ifdef ENABLE_NLS
 #include <libintl.h>
 #endif
@@ -122,6 +83,9 @@ void FaxListDialogCB::fax_list_button_clicked(GtkWidget* widget_p, void* data) {
     instance_p->reset_sig();
     instance_p->display_new_fax_count();
   }
+  else if (widget_p == instance_p->page_setup_button_p) {
+    MonoTiffPrintManager::page_setup(instance_p->get_win());
+  }
   else {
     write_error("Callback error in FaxListDialogCB::fax_list_book_button_clicked()\n");
     instance_p->close();
@@ -181,22 +145,55 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
   fax_list_manager.insert_fax_tree_view(GTK_CONTAINER(fax_list_scrolled_window_p));
 
   // set up the tool bar
+#if GTK_CHECK_VERSION(2,16,0)
+  gtk_orientable_set_orientation(GTK_ORIENTABLE(toolbar_p), GTK_ORIENTATION_HORIZONTAL);
+#else
   gtk_toolbar_set_orientation(toolbar_p, GTK_ORIENTATION_HORIZONTAL);
+#endif
   gtk_toolbar_set_style(toolbar_p, GTK_TOOLBAR_ICONS);
-  gtk_toolbar_set_tooltips(toolbar_p, true);
-
-  // is this necessary after calling gtk_toolbar_set_tooltips()?
-  // this is OK as tooltips is a public member of GtkToolbar
-  gtk_tooltips_enable(toolbar_p->tooltips);
 
   // bring up the icon size registered in MainWindow::MainWindow()
   GtkIconSize efax_gtk_button_size = gtk_icon_size_from_name("EFAX_GTK_BUTTON_SIZE");
 
-  GtkWidget* image_p;
+  GtkWidget* image_p = 0;
+  GtkIconTheme* icon_theme_p = gtk_icon_theme_get_default();
+  IconInfoScopedHandle icon_info_h(gtk_icon_theme_lookup_icon(icon_theme_p,
+							      "document-page-setup",
+							      22, GtkIconLookupFlags(0)));
+  if (icon_info_h.get()) {
+    const gchar* icon_path_p = gtk_icon_info_get_filename(icon_info_h.get());
+    if (icon_path_p) {
+      GError* error_p = 0;
+      GobjHandle<GdkPixbuf> pixbuf_h(gdk_pixbuf_new_from_file(icon_path_p, &error_p));
+      if (pixbuf_h.get()) {
+	image_p = gtk_image_new_from_pixbuf(pixbuf_h);
+      }
+      else {
+	write_error("Pixbuf error in FaxList::FaxList()\n");
+	if (error_p) {
+	  GerrorScopedHandle handle_h(error_p);
+	  write_error(error_p->message);
+	  write_error("\n");
+	}
+      }
+    }
+  }
+
+  if (!image_p) {
+    image_p = gtk_image_new_from_stock(GTK_STOCK_PREFERENCES, efax_gtk_button_size);
+  }
+  page_setup_button_p = gtk_button_new();
+  gtk_container_add(GTK_CONTAINER(page_setup_button_p), image_p);
+  toolbar_append_widget(toolbar_p, page_setup_button_p,
+			gettext("Print page setup"));
+  set_page_setup_button();
+  g_signal_connect(G_OBJECT(page_setup_button_p), "clicked",
+		   G_CALLBACK(FaxListDialogCB::fax_list_button_clicked), this);
+
   image_p = gtk_image_new_from_stock(GTK_STOCK_PRINT, efax_gtk_button_size);
   gtk_container_add(GTK_CONTAINER(print_button_p), image_p);
   toolbar_append_widget(toolbar_p, print_button_p,
-			gettext("Print selected fax"), "");
+			gettext("Print selected fax"));
 
   gtk_widget_set_sensitive(print_button_p, false);
   gtk_button_set_relief(GTK_BUTTON(print_button_p), GTK_RELIEF_NONE);
@@ -208,7 +205,7 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
     gtk_container_add(GTK_CONTAINER(view_button_p), image_p);
   }
   toolbar_append_widget(toolbar_p, view_button_p,
-			gettext("View selected fax"), "");
+			gettext("View selected fax"));
   gtk_widget_set_sensitive(view_button_p, false);
   gtk_button_set_relief(GTK_BUTTON(view_button_p), GTK_RELIEF_NONE);
 
@@ -219,21 +216,21 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
     gtk_container_add(GTK_CONTAINER(describe_button_p), image_p);
   }
   toolbar_append_widget(toolbar_p, describe_button_p,
-			gettext("Add/amend description for selected fax"), "");
+			gettext("Add/amend description for selected fax"));
   gtk_widget_set_sensitive(describe_button_p, false);
   gtk_button_set_relief(GTK_BUTTON(describe_button_p), GTK_RELIEF_NONE);
 
   image_p = gtk_image_new_from_stock(GTK_STOCK_DELETE, efax_gtk_button_size);
   gtk_container_add(GTK_CONTAINER(delete_fax_button_p), image_p);
   toolbar_append_widget(toolbar_p, delete_fax_button_p,
-			gettext("Delete selected fax"), "");
+			gettext("Delete selected fax"));
   gtk_widget_set_sensitive(delete_fax_button_p, false);
   gtk_button_set_relief(GTK_BUTTON(delete_fax_button_p), GTK_RELIEF_NONE);
 
   image_p = gtk_image_new_from_stock(GTK_STOCK_CLEAR, efax_gtk_button_size);
   gtk_container_add(GTK_CONTAINER(empty_trash_button_p), image_p);
   toolbar_append_widget(toolbar_p, empty_trash_button_p,
-			gettext("Empty trash folder"), "");
+			gettext("Empty trash folder"));
   gtk_widget_set_sensitive(empty_trash_button_p, false);
   gtk_button_set_relief(GTK_BUTTON(empty_trash_button_p), GTK_RELIEF_NONE);
 
@@ -244,7 +241,7 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
     gtk_container_add(GTK_CONTAINER(add_folder_button_p), image_p);
   }
   toolbar_append_widget(toolbar_p, add_folder_button_p,
-			gettext("Add new folder"), "");
+			gettext("Add new folder"));
   gtk_widget_set_sensitive(add_folder_button_p, true);
   gtk_button_set_relief(GTK_BUTTON(add_folder_button_p), GTK_RELIEF_NORMAL);
 
@@ -255,7 +252,7 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
     gtk_container_add(GTK_CONTAINER(delete_folder_button_p), image_p);
   }
   toolbar_append_widget(toolbar_p, delete_folder_button_p,
-			gettext("Delete selected folder"), "");
+			gettext("Delete selected folder"));
   gtk_widget_set_sensitive(delete_folder_button_p, false);
   gtk_button_set_relief(GTK_BUTTON(delete_folder_button_p), GTK_RELIEF_NONE);
 
@@ -310,7 +307,7 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
     image_p = gtk_image_new_from_stock(GTK_STOCK_UNDO, efax_gtk_button_size);
     gtk_container_add(GTK_CONTAINER(reset_button_p), image_p);
     toolbar_append_widget(toolbar_p, reset_button_p,
-			  gettext("Reset new faxes count"), "");
+			  gettext("Reset new faxes count"));
     gtk_widget_set_sensitive(reset_button_p, true);
     gtk_button_set_relief(GTK_BUTTON(reset_button_p), GTK_RELIEF_NORMAL);
     g_signal_connect(G_OBJECT(reset_button_p), "clicked",
@@ -324,17 +321,11 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
     gtk_container_add(GTK_CONTAINER(new_fax_frame_p), new_fax_label_p);
     gtk_label_set_justify(GTK_LABEL(new_fax_label_p), GTK_JUSTIFY_RIGHT);
 
-#if GTK_CHECK_VERSION(2,4,0)
     GtkToolItem* separator_p = gtk_separator_tool_item_new();
     gtk_separator_tool_item_set_draw(GTK_SEPARATOR_TOOL_ITEM(separator_p), false);
     gtk_tool_item_set_expand(separator_p, true);
     gtk_toolbar_insert(GTK_TOOLBAR(toolbar_p), separator_p, -1);
-    toolbar_append_widget(GTK_TOOLBAR(toolbar_p), new_fax_frame_p, "", "");
-#else
-    GtkWidget* new_fax_toolbar_p = gtk_toolbar_new();
-    gtk_box_pack_end(GTK_BOX(toolbar_box_p), new_fax_toolbar_p, false, false, 0);
-    toolbar_append_widget(GTK_TOOLBAR(new_fax_toolbar_p), new_fax_frame_p, "", "");
-#endif
+    toolbar_append_widget(GTK_TOOLBAR(toolbar_p), new_fax_frame_p, "");
 
     // as this is in the constructor, get_new_fax_count() always shows a 0 count
     // but this is OK as we are using it here to size the label - the user must
@@ -349,9 +340,16 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
   gtk_box_pack_start(GTK_BOX(fax_list_box_p), GTK_WIDGET(table_p), true, true, 0);
   
   // now connect up the signal which indicates a selection has been made
-  fax_list_manager.selection_notify.connect(sigc::mem_fun(*this, &FaxListDialog::set_buttons_slot));
+  fax_list_manager.selection_notify.connect(Callback::make(*this, &FaxListDialog::set_buttons_cb));
 
+  // and connect up the signal for a double click on a row in the fax tree view
+  fax_list_manager.fax_double_click_notify.connect(Callback::make(*this, &FaxListDialog::view_fax));
+
+#if GTK_CHECK_VERSION(2,20,0)
+  gtk_widget_set_can_default(close_button_p, true);
+#else
   GTK_WIDGET_SET_FLAGS(close_button_p, GTK_CAN_DEFAULT);
+#endif
   gtk_widget_grab_focus(close_button_p);
 
   gtk_container_set_border_width(GTK_CONTAINER(table_p), standard_size/3);
@@ -365,12 +363,12 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
   gtk_container_add(GTK_CONTAINER(get_win()), GTK_WIDGET(fax_list_box_p));
 
   gtk_window_set_default_size(get_win(), standard_size * 18, standard_size * 16);
-#if GTK_CHECK_VERSION(2,4,0)
+
   gtk_toolbar_set_show_arrow(GTK_TOOLBAR(toolbar_p), false);
-#endif
+
   gtk_widget_show_all(GTK_WIDGET(get_win()));
 
-  // not needed if gtk_toolbar_set_show_arrow() has been set false, or GTK+ version < 2.4
+  // not needed if gtk_toolbar_set_show_arrow() has been set false
   /*
   if (mode == FaxListEnum::received) {
     gtk_widget_set_size_request(GTK_WIDGET(toolbar_p),
@@ -403,7 +401,7 @@ FaxListDialog::~FaxListDialog(void) {
   else is_fax_sent_list--;
 }
 
-void FaxListDialog::set_buttons_slot(void) {
+void FaxListDialog::set_buttons_cb(void) {
 
   // see if fax is selected
   RowPathList::size_type fax_count = fax_list_manager.is_fax_selected();
@@ -457,6 +455,23 @@ void FaxListDialog::set_buttons_slot(void) {
   }
 }
 
+// we have this as a separate method from set_buttons_cb, as whether
+// the page setup button is sensitive depends on a global setting, not
+// on selected faxes (it depends on whether the GTK+ print system
+// setting has been chosen).  It is called whenever the settings dialog
+// emits the 'accepted' signal, via MainWindow::settings_changed_cb().
+void FaxListDialog::set_page_setup_button(void) {
+
+  if (prog_config.gtkprint) {
+    gtk_widget_set_sensitive(page_setup_button_p, true);
+    gtk_button_set_relief(GTK_BUTTON(page_setup_button_p), GTK_RELIEF_NORMAL);
+  }
+  else {
+    gtk_widget_set_sensitive(page_setup_button_p, false);
+    gtk_button_set_relief(GTK_BUTTON(page_setup_button_p), GTK_RELIEF_NONE);
+  }
+}
+
 void FaxListDialog::delete_fax(void) {
 
   RowPathList::size_type fax_count;
@@ -479,7 +494,7 @@ void FaxListDialog::delete_fax(void) {
 
       PromptDialog* dialog_p = new PromptDialog(msg.c_str(), gettext("efax-gtk: Delete fax"),
 						standard_size, get_win());
-      dialog_p->accepted.connect(sigc::mem_fun(fax_list_manager, &FaxListManager::delete_fax));
+      dialog_p->accepted.connect(Callback::make(fax_list_manager, &FaxListManager::delete_fax));
       // there is no memory leak -- the memory will be deleted when PromptDialog closes
     }
     else fax_list_manager.move_selected_faxes_to_trash_folder();
@@ -497,7 +512,7 @@ void FaxListDialog::describe_fax_prompt(void) {
     DescriptionDialog* dialog_p = new DescriptionDialog(standard_size,
 							description.c_str(),
 							get_win());
-    dialog_p->accepted.connect(sigc::mem_fun(fax_list_manager, &FaxListManager::describe_fax));
+    dialog_p->accepted.connect(Callback::make(fax_list_manager, &FaxListManager::describe_fax));
     // there is no memory leak -- the memory will be deleted when DescriptionDialog closes
   }
 }
@@ -509,14 +524,14 @@ void FaxListDialog::empty_trash_prompt(void) {
 			  "in the Trash folder from the file system)"));
   PromptDialog* dialog_p = new PromptDialog(msg.c_str(), gettext("efax-gtk: Trash folder"),
 					    standard_size, get_win());
-  dialog_p->accepted.connect(sigc::mem_fun(fax_list_manager, &FaxListManager::empty_trash_folder));
+  dialog_p->accepted.connect(Callback::make(fax_list_manager, &FaxListManager::empty_trash_folder));
   // there is no memory leak -- the memory will be deleted when PromptDialog closes
 }
 
 void FaxListDialog::add_folder_prompt(void) {
 
   AddFolderDialog* dialog_p = new AddFolderDialog(standard_size, get_win());
-  dialog_p->accepted.connect(sigc::mem_fun(*this, &FaxListDialog::add_folder));
+  dialog_p->accepted.connect(Callback::make(*this, &FaxListDialog::add_folder));
   // there is no memory leak -- the memory will be deleted when AddFolderDialog closes
 
 }
@@ -564,7 +579,7 @@ void FaxListDialog::delete_folder_prompt(void) {
 
       PromptDialog* dialog_p = new PromptDialog(msg.c_str(), gettext("efax-gtk: Delete folder"),
 						standard_size, get_win());
-      dialog_p->accepted.connect(sigc::mem_fun(fax_list_manager, &FaxListManager::delete_folder));
+      dialog_p->accepted.connect(Callback::make(fax_list_manager, &FaxListManager::delete_folder));
       // there is no memory leak -- the memory will be deleted when PromptDialog closes
     }
   }
@@ -583,16 +598,12 @@ void FaxListDialog::print_fax_prompt(void) {
     // bool which can be safely concurrently read in different threads and in any
     // event it is read in FaxListDialog::print_fax_thread() in a separate process
     // after fork()ing so there isn't even concurrent access
-#if GTK_CHECK_VERSION(2,10,0)
     if (prog_config.gtkprint) print_fax();
     else if (prog_config.print_popup) {
-#else
-    if (prog_config.print_popup) {
-#endif      
       std::string msg(gettext("Print selected fax?"));
       PromptDialog* dialog_p = new PromptDialog(msg.c_str(), gettext("efax-gtk: Print fax"),
 						standard_size, get_win());
-      dialog_p->accepted.connect(sigc::mem_fun(*this, &FaxListDialog::print_fax));
+      dialog_p->accepted.connect(Callback::make(*this, &FaxListDialog::print_fax));
       // there is no memory leak -- the memory will be deleted when PromptDialog closes
     }
     else print_fax();
@@ -684,7 +695,6 @@ std::pair<const char*, char* const*> FaxListDialog::get_fax_to_ps_parms
     temp += prog_config.page_dim;
     efix_parms.push_back(temp);
 
-#ifdef HAVE_STRINGSTREAM
     if (allow_shrink && prog_config.print_shrink.compare("100")) { // if print_shrink is not 100
       temp = "-s0.";
       temp += prog_config.print_shrink;
@@ -692,9 +702,9 @@ std::pair<const char*, char* const*> FaxListDialog::get_fax_to_ps_parms
 
       std::ostringstream strm;
 
-#  ifdef HAVE_STREAM_IMBUE
+#ifdef HAVE_STREAM_IMBUE
       strm.imbue(std::locale::classic());
-#  endif // HAVE_STREAM_IMBUE
+#endif // HAVE_STREAM_IMBUE
 
       if (!prog_config.page_size.compare("a4")) {
 
@@ -733,9 +743,9 @@ std::pair<const char*, char* const*> FaxListDialog::get_fax_to_ps_parms
     int partnumber = 1;
     std::ostringstream strm;
 
-#  ifdef HAVE_STREAM_IMBUE
+#ifdef HAVE_STREAM_IMBUE
     strm.imbue(std::locale::classic());
-#  endif // HAVE_STREAM_IMBUE
+#endif // HAVE_STREAM_IMBUE
 
     strm << basename.c_str() << std::setfill('0')
 	 << std::setw(3) << partnumber;
@@ -750,89 +760,6 @@ std::pair<const char*, char* const*> FaxListDialog::get_fax_to_ps_parms
 	   << std::setw(3) << partnumber;
       result = access(strm.str().c_str(), R_OK);
     }
-
-#else
-    if (allow_shrink && prog_config.print_shrink.compare("100")) { // if print_shrink is not 100
-      temp = "-s0.";
-      temp += prog_config.print_shrink;
-      efix_parms.push_back(temp);
-
-      std::ostrstream strm;
-
-#  ifdef HAVE_STREAM_IMBUE
-      strm.imbue(std::locale::classic());
-#  endif // HAVE_STREAM_IMBUE
-
-      if (!prog_config.page_size.compare("a4")) {
-
-	strm << "-d";
-	float val = 210 * (100 - std::atoi(prog_config.print_shrink.c_str()))/200.0;
-	strm << static_cast<int>(val + 0.5);
-	val = 297 * (100 - std::atoi(prog_config.print_shrink.c_str()))/200.0;
-	strm << ',' << static_cast<int>(val + 0.5) << "mm" << std::ends;
-
-	const char* dim = strm.str();
-	efix_parms.push_back(dim);
-	delete[] dim;
-      }
-    
-      else if (!prog_config.page_size.compare("letter")) {
-
-	strm << "-d";
-	float val = 216 * (100 - std::atoi(prog_config.print_shrink.c_str()))/200.0;
-	strm << static_cast<int>(val + 0.5);
-	val = 279 * (100 - std::atoi(prog_config.print_shrink.c_str()))/200.0;
-	strm << ',' << static_cast<int>(val + 0.5) << "mm" << std::ends;
-
-	const char* dim = strm.str();
-	efix_parms.push_back(dim);
-	delete[] dim;
-      }
-
-      else if (!prog_config.page_size.compare("legal")) {
-
-	strm << "-d";
-	float val = 216 * (100 - std::atoi(prog_config.print_shrink.c_str()))/200.0;
-	strm << static_cast<int>(val + 0.5);
-	val = 356 * (100 - std::atoi(prog_config.print_shrink.c_str()))/200.0;
-	strm << ',' << static_cast<int>(val + 0.5) << "mm" << std::ends;
-
-      const char* dim = strm.str();
-      efix_parms.push_back(dim);
-      delete[] dim;
-      }
-    }
-
-    int partnumber = 1;
-    std::ostrstream strm;
-
-#  ifdef HAVE_STREAM_IMBUE
-    strm.imbue(std::locale::classic());
-#  endif // HAVE_STREAM_IMBUE
-
-    strm << basename.c_str() << std::setfill('0')
-	 << std::setw(3) << partnumber << std::ends;
-    const char* file_name = strm.str();
-    int result = access(file_name, R_OK);
-
-    while (!result) {
-      efix_parms.push_back(file_name);
-      delete[] file_name;
-
-      partnumber++;
-      std::ostrstream strm;
-
-#  ifdef HAVE_STREAM_IMBUE
-      strm.imbue(std::locale::classic());
-#  endif // HAVE_STREAM_IMBUE
-
-      strm << basename.c_str() << std::setfill('0')
-	   << std::setw(3) << partnumber << std::ends;
-      file_name = strm.str();
-      result = access(file_name, R_OK);
-    }
-    delete[] file_name;
-#endif
   } // end of mutex lock scope block
 
   char** exec_parms = new char*[efix_parms.size() + 1];
@@ -957,11 +884,7 @@ void FaxListDialog::print_fax(void) {
     *fax_basename_a += '.';
 
     // create the print manager
-#if GTK_CHECK_VERSION(2,10,0)
-    if (prog_config.gtkprint) async_queue.push(FilePrintManager::create_manager(get_win(),
-										gettext("efax-gtk: Print fax"),
-										prog_config.window_icon_h));
-#endif
+    if (prog_config.gtkprint) async_queue.push(MonoTiffPrintManager::create_manager(get_win()));
 
     // now block off the signals for which we have set handlers so that the worker
     // thread does not receive the signals, otherwise we will have memory synchronisation
@@ -991,7 +914,7 @@ void FaxListDialog::print_fax(void) {
     if (!res.get()) {
       delete arg_p;
       write_error("Cannot start new print thread, fax will not be printed\n");
-#if GTK_CHECK_VERSION(2,10,0)
+
       // Because another thread (for another print job) could have been
       // launched this may extract a different print manager object from
       // the one created above in this thread, but that doesn't matter as
@@ -1006,7 +929,6 @@ void FaxListDialog::print_fax(void) {
 	  write_error(" in FaxListDialog::print_fax()\n");
 	}
       }
-#endif
     }
     // now unblock the signals so that the initial (GUI) thread can receive them
     pthread_sigmask(SIG_UNBLOCK, &sig_mask, 0);
@@ -1019,8 +941,7 @@ void FaxListDialog::print_fax_thread(std::string* arg_p, bool use_gtkprint) {
   const std::auto_ptr<std::string> fax_basename_a(arg_p);
 
   // extract any FilePrintManager object
-#if GTK_CHECK_VERSION(2,10,0)
-  IntrusivePtr<FilePrintManager> print_manager_i;
+  IntrusivePtr<MonoTiffPrintManager> print_manager_i;
   if (use_gtkprint) {
     try {
      async_queue.pop(print_manager_i);
@@ -1030,146 +951,112 @@ void FaxListDialog::print_fax_thread(std::string* arg_p, bool use_gtkprint) {
       write_error(" in FaxListDialog::print_fax_thread()\n");
       return;
     }
-  }
-#endif
-  // get the arguments for the exec() calls below (because this is a
-  // multi- threaded program, we must do this before fork()ing because
-  // we use functions to get the arguments which are not async-signal-safe)
-  std::pair<const char*, char* const*> fax_to_ps_parms(get_fax_to_ps_parms(*fax_basename_a, true));
 
-#if GTK_CHECK_VERSION(2,10,0)
-  if (use_gtkprint) {
+    std::vector<std::string> filenames;
 
-    // get a temporary file
-    std::string filename;
-    {
-      Thread::Mutex::Lock lock(working_dir_mutex);
-      filename = working_dir;
-    }
-    filename += "/efax-gtk-print-XXXXXX";
+    int partnumber = 1;
+    std::ostringstream strm;
 
-    std::string::size_type size = filename.size() + 1;
-    ScopedHandle<char*> tempfile_h(new char[size]);
-    std::memcpy(tempfile_h.get(), filename.c_str(), size); // this will include the terminating '\0' in the copy
-    int file_fd = mkstemp(tempfile_h.get());
+#ifdef HAVE_STREAM_IMBUE
+    strm.imbue(std::locale::classic());
+#endif // HAVE_STREAM_IMBUE
 
-    if (file_fd == -1) {
-      write_error("Failed to make temporary file for printing");
-      return;
+    strm << fax_basename_a->c_str() << std::setfill('0')
+	 << std::setw(3) << partnumber;
+    int result = access(strm.str().c_str(), R_OK);
+
+    while (!result) {
+      filenames.push_back(strm.str());
+	
+      partnumber++;
+      strm.str("");
+      strm << fax_basename_a->c_str() << std::setfill('0')
+	   << std::setw(3) << partnumber;
+      result = access(strm.str().c_str(), R_OK);
     }
 
-    // now create a pipe and proceed to fork to write the postscript to temporary file
-    PipeFifo fork_pipe(PipeFifo::block);
-
-    // now fork to create the process which will write postscript to stdout
-    pid_t pid = fork();
-
-    if (pid == -1) {
-      write_error("Fork error - exiting\n");
-      std::exit(FORK_ERROR);
-    }
-  
-    if (!pid) { // child process which will send postscript to stdout
-  
-      // this is the child process to write to stdin
-      // now we have fork()ed we can connect to stderr
-      connect_to_stderr();
-
-      fork_pipe.connect_to_stdout();
-      execvp(fax_to_ps_parms.first, fax_to_ps_parms.second);
-
-      // if we reached this point, then the execvp() call must have failed
-      // report error and then end process - use _exit(), not exit()
-      write_error("Can't find the efix-0.9a program - please check your installation\n"
-		  "and the PATH environmental variable\n");
-      _exit(0);
-    }
-
-    // write from the pipe (containing the generated PS text) to file
-    fork_pipe.make_readonly();
-    write_pipe_to_file(&fork_pipe, file_fd);
-    while (::close(file_fd) == -1 && errno == EINTR);
-
-    print_manager_i->set_filename(tempfile_h.get());
+    print_manager_i->set_filenames(filenames);
     print_manager_i->print(); // this will pass ownership to the FilePrintManager object
                               // which will control its own destiny now, via the GTK+
                               // print system
   }
   else {
-#endif
 
-  std::pair<const char*, char* const*> print_from_stdin_parms(get_print_from_stdin_parms());
-  if (print_from_stdin_parms.first) { // this will be 0 if get_print_from_stdin_parms()
-                                      // threw a Utf8::ConversionError)
+    // get the arguments for the exec() calls below (because this is a
+    // multi- threaded program, we must do this before fork()ing because
+    // we use functions to get the arguments which are not async-signal-safe)
+    std::pair<const char*, char* const*> fax_to_ps_parms(get_fax_to_ps_parms(*fax_basename_a, true));
 
-    // now launch a new process to control the printing process
-    // the main program process needs to continue while the printing
-    // is going on
-    pid_t pid = fork();
+    std::pair<const char*, char* const*> print_from_stdin_parms(get_print_from_stdin_parms());
+    if (print_from_stdin_parms.first) { // this will be 0 if get_print_from_stdin_parms()
+                                        // threw a Utf8::ConversionError)
 
-    if (pid == -1) {
-      write_error("Fork error - exiting\n");
-      std::exit(FORK_ERROR);
-    }
-    if (!pid) { // child print process
-
-      // make the call to connect_to_stderr() in this child process
-      // dependent on calling a print confirmation popup dialog: if we have
-      // elected not to bring up such a dialog, we are probably using a
-      // print manager such as kprinter, and we can get confusing messages
-      // from stderr from these
-      // no mutex is required for the access to prog_config.print_popup
-      // because we have fork()ed into a separate process
-      if (prog_config.print_popup) connect_to_stderr();
-
-      // now create a blocking pipe which the print processes can use to
-      // communicate
-      PipeFifo fork_pipe;
-      try {
-	fork_pipe.open(PipeFifo::block);
-      }
-      catch (PipeError&) {
-	write_error("Cannot open pipe in FaxListDialog::print_fax_thread()\n");
-	_exit(PIPE_ERROR);
-      }
-      
+      // now launch a new process to control the printing process
+      // the main program process needs to continue while the printing
+      // is going on
       pid_t pid = fork();
 
       if (pid == -1) {
 	write_error("Fork error - exiting\n");
-	_exit(FORK_ERROR);
+	std::exit(FORK_ERROR);
       }
-      if (!pid) { // process to exec to print_from_stdin()
+      if (!pid) { // child print process
+
+	// make the call to connect_to_stderr() in this child process
+	// dependent on calling a print confirmation popup dialog: if we have
+	// elected not to bring up such a dialog, we are probably using a
+	// print manager such as kprinter, and we can get confusing messages
+	// from stderr from these
+	// no mutex is required for the access to prog_config.print_popup
+	// because we have fork()ed into a separate process
+	if (prog_config.print_popup) connect_to_stderr();
+
+	// now create a blocking pipe which the print processes can use to
+	// communicate
+	PipeFifo fork_pipe;
+	try {
+	  fork_pipe.open(PipeFifo::block);
+	}
+	catch (PipeError&) {
+	  write_error("Cannot open pipe in FaxListDialog::print_fax_thread()\n");
+	  _exit(PIPE_ERROR);
+	}
       
-	fork_pipe.connect_to_stdin();
-	execvp(print_from_stdin_parms.first, print_from_stdin_parms.second);
+	pid_t pid = fork();
+
+	if (pid == -1) {
+	  write_error("Fork error - exiting\n");
+	  _exit(FORK_ERROR);
+	}
+	if (!pid) { // process to exec to print_from_stdin()
+
+	  fork_pipe.connect_to_stdin();
+	  execvp(print_from_stdin_parms.first, print_from_stdin_parms.second);
+	  // if we reached this point, then the execvp() call must have failed
+	  // write error and then end process - use _exit(), not exit()
+	  write_error("Can't find the print program - please check your installation\n"
+		      "and the PATH environmental variable\n");
+	  _exit(0);
+	}
+
+	// this is the process which will send postscript to stdout
+	fork_pipe.connect_to_stdout();
+	execvp(fax_to_ps_parms.first, fax_to_ps_parms.second);
+
 	// if we reached this point, then the execvp() call must have failed
-	// write error and then end process - use _exit(), not exit()
-	write_error("Can't find the print program - please check your installation\n"
+	// report error and then end process - use _exit(), not exit()
+	write_error("Can't find the efix-0.9a program - please check your installation\n"
 		    "and the PATH environmental variable\n");
 	_exit(0);
       }
-
-      // this is the process which will send postscript to stdout
-      fork_pipe.connect_to_stdout();
-      execvp(fax_to_ps_parms.first, fax_to_ps_parms.second);
-
-      // if we reached this point, then the execvp() call must have failed
-      // report error and then end process - use _exit(), not exit()
-      write_error("Can't find the efix-0.9a program - please check your installation\n"
-		  "and the PATH environmental variable\n");
-      _exit(0);
+      // release the memory allocated on the heap for
+      // the redundant print_from_stdin_parms and fax_to_ps_parms
+      // we are in the main parent process here - no worries about
+      // only being able to use async-signal-safe functions
+      delete_parms(print_from_stdin_parms);
     }
-    // release the memory allocated on the heap for
-    // the redundant print_from_stdin_parms and fax_to_ps_parms
-    // we are in the main parent process here - no worries about
-    // only being able to use async-signal-safe functions
-    delete_parms(print_from_stdin_parms);
+    delete_parms(fax_to_ps_parms);
   }
-#if GTK_CHECK_VERSION(2,10,0)
-  }
-#endif
-  delete_parms(fax_to_ps_parms);
 }
 
 
@@ -1367,7 +1254,9 @@ void FaxListDialog::display_new_fax_count(void) {
 
   // if mode == FaxListEnum::sent we don't do anything at the moment
   if (mode == FaxListEnum::received) {
-    int fax_count = get_new_fax_count_sig();
+    int fax_count = 0; // make the count 0 if the emitter has not
+                       // yet been connected up
+    get_new_fax_count_sig(fax_count);
     
     // Note: if fax_count == 0 we could pass NULL as the colour argument to
     // gtk_widget_modify_fg() to bring up the default colour, but unfortunately
@@ -1383,19 +1272,10 @@ void FaxListDialog::display_new_fax_count(void) {
     // used by gtk_widget_modify_fg() - that function will do it itself
     gtk_widget_modify_fg(new_fax_label_p, GTK_STATE_NORMAL, &fg_colour);
 
-#ifdef HAVE_STRINGSTREAM
     std::ostringstream strm;
     strm << gettext("New faxes:");
     strm << std::setfill(' ') << std::setw(4) << fax_count;
     gtk_label_set_text(GTK_LABEL(new_fax_label_p), strm.str().c_str());
-#else
-    std::ostrstream strm;
-    strm << gettext("New faxes: ");
-    strm << std::setfill(' ') << std::setw(4) << fax_count << std::ends;
-    const char* text = strm.str();
-    gtk_label_set_text(GTK_LABEL(new_fax_label_p), text);
-    delete[] text;
-#endif
   }
 }
 
@@ -1428,8 +1308,11 @@ void EntryDialogCB::entry_dialog_selected(GtkWidget* widget_p, void* data) {
 gboolean EntryDialogCB::entry_dialog_key_press_event(GtkWidget*, GdkEventKey* event_p, void* data) {
   EntryDialog* instance_p = static_cast<EntryDialog*>(data);
   int keycode = event_p->keyval;
+#if GTK_CHECK_VERSION(2,20,0)
+  bool cancel_focus = gtk_widget_has_focus(instance_p->cancel_button_p);
+#else
   bool cancel_focus = GTK_WIDGET_HAS_FOCUS(instance_p->cancel_button_p);
-
+#endif
   if (keycode == GDK_Return && !cancel_focus) {
     if (instance_p->selected_impl()) instance_p->close();
     return true;  // stop processing here
@@ -1454,7 +1337,7 @@ EntryDialog::EntryDialog(const int standard_size, const char* entry_text,
   GtkTable* table_p = GTK_TABLE(gtk_table_new(3, 1, false));
 
   gtk_button_box_set_layout(GTK_BUTTON_BOX(button_box_p), GTK_BUTTONBOX_END);
-  gtk_button_box_set_spacing(GTK_BUTTON_BOX(button_box_p), standard_size/2);
+  gtk_box_set_spacing(GTK_BOX(button_box_p), standard_size/2);
 
   gtk_container_add(GTK_CONTAINER(button_box_p), cancel_button_p);
   gtk_container_add(GTK_CONTAINER(button_box_p), ok_button_p);
@@ -1481,9 +1364,13 @@ EntryDialog::EntryDialog(const int standard_size, const char* entry_text,
   g_signal_connect(G_OBJECT(get_win()), "key_press_event",
 		   G_CALLBACK(EntryDialogCB::entry_dialog_key_press_event), this);
 
-
+#if GTK_CHECK_VERSION(2,20,0)
+  gtk_widget_set_can_default(ok_button_p, true);
+  gtk_widget_set_can_default(cancel_button_p, true);
+#else
   GTK_WIDGET_SET_FLAGS(ok_button_p, GTK_CAN_DEFAULT);
   GTK_WIDGET_SET_FLAGS(cancel_button_p, GTK_CAN_DEFAULT);
+#endif
 
   gtk_container_add(GTK_CONTAINER(get_win()), GTK_WIDGET(table_p));
   
