@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2007 Chris Vine
+/* Copyright (C) 2001 to 2007 and 2009 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -11,30 +11,23 @@ COPYING distributed with the source files.
 
 #include "prog_defs.h"
 
-// something in gtkprintjob.h clashes with the C++ headers with
-// libstdc++-5 and it needs to be included here (via print_manager.h)
-// to obtain a clean compile
-#include <gtk/gtkversion.h>
-#if GTK_CHECK_VERSION(2,10,0)
-#include "utils/file_print_manager.h"
-#include "utils/intrusive_ptr.h"
-#include "utils/async_queue.h"
-#endif
-
 #include <string>
 #include <utility>
 #include <vector>
 
-#include <gtk/gtkwidget.h>
-#include <gtk/gtkwindow.h>
-
-#include <sigc++/sigc++.h>
+#include <gtk/gtk.h>
 
 #include "fax_list_manager.h"
-#include "utils/window.h"
-#include "utils/pipes.h"
-#include "utils/mutex.h"
-#include "utils/shared_handle.h"
+#include "utils/mono_tiff_print_manager.h"
+
+#include <c++-gtk-utils/pipes.h>
+#include <c++-gtk-utils/window.h>
+#include <c++-gtk-utils/emitter.h>
+#include <c++-gtk-utils/mutex.h>
+#include <c++-gtk-utils/shared_handle.h>
+#include <c++-gtk-utils/intrusive_ptr.h>
+#include <c++-gtk-utils/async_queue.h>
+
 
 namespace { // we put the function in anonymous namespace in fax_list.cpp
             // so it is not exported at link time
@@ -43,7 +36,7 @@ namespace FaxListDialogCB {
 }
 }
 
-class FaxListDialog: public sigc::trackable, public WinBase {
+class FaxListDialog: public WinBase {
   FaxListEnum::Mode mode;
   static int is_fax_received_list;
   static int is_fax_sent_list;
@@ -55,12 +48,10 @@ class FaxListDialog: public sigc::trackable, public WinBase {
   std::vector<SharedHandle<char*> > view_files;
 
   FaxListManager fax_list_manager;
-
-#if GTK_CHECK_VERSION(2,10,0)
-  AsyncQueue<IntrusivePtr<FilePrintManager> > async_queue;
-#endif
+  AsyncQueue<IntrusivePtr<MonoTiffPrintManager> > async_queue;
 
   GtkWidget* close_button_p;
+  GtkWidget* page_setup_button_p;
   GtkWidget* print_button_p;
   GtkWidget* view_button_p;
   GtkWidget* describe_button_p;
@@ -72,7 +63,7 @@ class FaxListDialog: public sigc::trackable, public WinBase {
 
   GtkWidget* new_fax_label_p;
 
-  void set_buttons_slot(void);
+  void set_buttons_cb(void);
   void describe_fax_prompt(void);
   void delete_fax(void);
   void add_folder_prompt(void);
@@ -92,18 +83,20 @@ class FaxListDialog: public sigc::trackable, public WinBase {
 public:
   friend void FaxListDialogCB::fax_list_button_clicked(GtkWidget*, void*);
 
-  sigc::signal0<int> get_new_fax_count_sig;
-  sigc::signal0<void> reset_sig;
+  Releaser releaser;
+  EmitterArg<int&> get_new_fax_count_sig;
+  Emitter reset_sig;
 
   // display_new_fax_count() calls get_new_fax_count_sig, so until it has been
   // connected to a function which returns this, it will show 0 as the count
   void display_new_fax_count(void);
   static int get_is_fax_received_list(void) {return is_fax_received_list;}
   static int get_is_fax_sent_list(void) {return is_fax_sent_list;}
-  void insert_new_fax_slot(const std::pair<std::string, std::string>& fax_info) {
+  void insert_new_fax_cb(const std::pair<std::string, std::string>& fax_info) {
     fax_list_manager.insert_new_fax_in_base(fax_info.first, fax_info.second);
     display_new_fax_count();
   }
+  void set_page_setup_button(void);
 
   FaxListDialog(FaxListEnum::Mode, const int standard_size_);
   ~FaxListDialog(void);
@@ -131,7 +124,7 @@ public:
   friend void EntryDialogCB::entry_dialog_selected(GtkWidget*, void*);
   friend gboolean EntryDialogCB::entry_dialog_key_press_event(GtkWidget*, GdkEventKey*, void*);
 
-  sigc::signal1<void, const std::string&> accepted;
+  EmitterArg<const std::string&> accepted;
   EntryDialog(const int standard_size, const char* entry_text,
 	      const char* caption, const char* label_text,
 	      GtkWindow* parent_p);
