@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2006 Chris Vine
+/* Copyright (C) 2001 to 2010 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -8,6 +8,8 @@ COPYING distributed with the source files.
 
 #include <vector>
 #include <cstring>
+#include <ios>
+#include <ostream>
 #include <ctime>
 
 #include <unistd.h>
@@ -16,27 +18,12 @@ COPYING distributed with the source files.
 #include <dirent.h>
 #include <errno.h>
 
-#include <gtk/gtkliststore.h>
-#include <gtk/gtktreestore.h>
-#include <gtk/gtkcellrenderer.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkcellrendererpixbuf.h>
-#include <gtk/gtktreeviewcolumn.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkversion.h>
-
-#include <gdk/gdkpixmap.h>
-#include <glib/gmain.h>
-
-#ifdef HAVE_OSTREAM
-#include <ios>
-#include <ostream>
-#endif
-
 #include "fax_list_manager.h"
 #include "fax_list_manager_icons.h"
 #include "utils/tree_path_handle.h"
-#include "utils/utf8_utils.h"
+
+#include <c++-gtk-utils/timeout.h>
+#include <c++-gtk-utils/convert.h>
 
 #ifdef ENABLE_NLS
 #include <libintl.h>
@@ -234,6 +221,22 @@ gboolean FaxListManagerCB::fax_tree_view_motion_notify(GtkWidget*,
   return false;
 }
 
+gboolean FaxListManagerCB::fax_tree_view_mouse_click(GtkWidget*,
+						      GdkEventButton* event_p, void* data) {
+  
+  FaxListManager* instance_p = static_cast<FaxListManager*>(data);
+
+  // we can use fax_drag_source_enabled as a proxy for whether the pointer
+  // is over a selectable (and so selected) row in the tree view when
+  // a double click occurs
+  if (event_p->type == GDK_2BUTTON_PRESS
+      && instance_p->fax_drag_source_enabled) {
+    instance_p->fax_double_click_notify();
+  }
+  return false;
+}
+
+
 void FaxListManagerCB::drag_data_get(GtkWidget*, GdkDragContext*,
 				     GtkSelectionData* selection_data_p,
 				     guint, guint, void*) {
@@ -252,8 +255,13 @@ void FaxListManagerCB::drag_data_get(GtkWidget*, GdkDragContext*,
 
   const std::string message(DRAG_KEY_STRING);
 
+#if GTK_CHECK_VERSION(2,14,0)
+  gtk_selection_data_set(selection_data_p, gtk_selection_data_get_target(selection_data_p),
+			 8, (const guchar*)message.c_str(), message.size() + 1);
+#else
   gtk_selection_data_set(selection_data_p, selection_data_p->target, 8,
 			 (const guchar*)message.c_str(), message.size() + 1);
+#endif
 }
 
 void FaxListManagerCB::drag_data_received(GtkWidget*, GdkDragContext* context_p,
@@ -265,9 +273,16 @@ void FaxListManagerCB::drag_data_received(GtkWidget*, GdkDragContext* context_p,
   bool success = false;
   std::string message;
 
+#if GTK_CHECK_VERSION(2,14,0)
+  if(gtk_selection_data_get_length(selection_data_p) >= 0
+     && gtk_selection_data_get_format(selection_data_p) == 8) {
+    message = (const char*)gtk_selection_data_get_data(selection_data_p);
+  }
+#else
   if(selection_data_p->length >= 0 && selection_data_p->format == 8) {
     message = (const char*)selection_data_p->data;
   }
+#endif
 
   if (message.find(DRAG_KEY_STRING) != std::string::npos) {
     
@@ -299,17 +314,12 @@ void FaxListManagerCB::folder_selected(GtkTreeSelection*, void* data) {
   FaxListManager* instance_p = static_cast<FaxListManager*>(data);
 
   // display_faxes() needs to be called first before selection_notify
-  // sigc::signal is emitted so that the correct fax tree model is in
-  // position before slot connected to the signal is executed
+  // emitter is emitted so that the correct fax tree model is in
+  // position before the functor connected to the emitter is executed
   instance_p->display_faxes();
   instance_p->selection_notify();
 }
 
-gboolean FaxListManagerCB::write_path_timer_event(void* data) {
-  static_cast<FaxListManager*>(data)->write_path();
-  return true; // we want a multi-shot timer
-}
-
 void FaxListManagerCB::toggle_sort_type(GtkTreeViewColumn*, void* data) {
   FaxListManager* instance_p = static_cast<FaxListManager*>(data);
   if (instance_p->list_sort_type[instance_p->mode] == GTK_SORT_ASCENDING) 
@@ -468,9 +478,14 @@ FaxListManager::FaxListManager(FaxListEnum::Mode mode_):
   g_signal_connect(G_OBJECT(selection_p), "changed",
 		   G_CALLBACK(FaxListManagerCB::folder_selected), this);
 
+  // capture a double click on the fax tree view for viewing a fax
+  gtk_widget_add_events(GTK_WIDGET(fax_tree_view_p), GDK_BUTTON_PRESS_MASK);
+  g_signal_connect(G_OBJECT(fax_tree_view_p), "button_press_event",
+		   G_CALLBACK(FaxListManagerCB::fax_tree_view_mouse_click), this);
+
   // set a timeout to call write_path() at 5 minute intervals
-  timer_tag = g_timeout_add(WRITE_PATH_TIMER_INTERVAL,
-			    FaxListManagerCB::write_path_timer_event, this);
+  timer_tag = start_timeout(WRITE_PATH_TIMER_INTERVAL,
+			    Callback::make(*this, &FaxListManager::write_path_timer_cb));
 }
 
 FaxListManager::~FaxListManager(void) {
@@ -614,13 +629,13 @@ void FaxListManager::populate_fax_list(void) {
 	      || std::strcmp(direntry->d_name, "current"))) {
 	// checking for "current" will prevent double entries if completion of a received
 	// fax occurs while we are scanning the faxin directory - it is transferred to its
-	// own directory in the EfaxController::timer_event() callback and that callback
+	// own directory in the EfaxController::child_ended_cb() callback and that callback
 	// cannot be invoked until this method has returned (note also the checks for
 	// FaxListManager::is_fax_received_list_main_iteration() and
-	// FaxListManager::is_fax_sent_list_main_iteration() in EfaxController::timer_event())
+	// FaxListManager::is_fax_sent_list_main_iteration() in EfaxController::child_ended_cb())
 	// likewise a completed sent fax will not be stored in a directory in the faxsent
 	// directory (if we are scanning that directory) until that is done via
-	// EfaxController::timer_event()
+	// EfaxController::child_ended_cb()
 
 	std::string fax_name(direntry->d_name);
 
@@ -1150,13 +1165,11 @@ void FaxListManager::move_folder(GtkTreeIter* dest_folder_row_iter_p,
       folder_row_to_fax_model_map.insert(FolderRowToFaxModelMap::value_type(new_row_ref_h,
 									    fax_model_h));
 
-#if GTK_CHECK_VERSION(2,2,0)
       // get the moved folder to show to its dropped level (do this before the call
       // to move_child_folders_for_level() or it may be invalidated)
       if (gtk_tree_path_up(new_path_h)) {
 	gtk_tree_view_expand_to_path(folder_tree_view_p, new_path_h);
       }
-#endif
 
       // now move all the children of the moved folder to rejoin their parent
       move_child_folders_for_level(&folder_drag_source_row_iter, &new_row_iter);
