@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2005 Chris Vine
+/* Copyright (C) 2001 to 2010 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -17,24 +17,17 @@ COPYING distributed with the source files.
 #include <fstream>
 #include <utility>
 
-#include <gtk/gtkwidget.h>
-#include <gtk/gtktreemodel.h>
-#include <gtk/gtktreeview.h>
-#include <gtk/gtktreeselection.h>
-#include <gtk/gtkcontainer.h>
-#include <gtk/gtkdnd.h>
-#include <gtk/gtkenums.h>
+#include <gtk/gtk.h>
+#include <gdk/gdk.h>
+#include <glib.h>
 
-#include <gdk/gdkpixbuf.h>
-#include <glib/gtypes.h>
-
-#include <sigc++/sigc++.h>
-
-#include "utils/shared_handle.h"
-#include "utils/gobj_handle.h"
 #include "utils/tree_row_reference_handle.h"
 #include "utils/selected_rows_handle.h"
 
+#include <c++-gtk-utils/gobj_handle.h>
+#include <c++-gtk-utils/shared_handle.h>
+#include <c++-gtk-utils/emitter.h>
+
 
 typedef std::map<TreeRowRefSharedHandle, GobjHandle<GtkTreeModel> > FolderRowToFaxModelMap;
 typedef std::map<std::string, TreeRowRefSharedHandle> PathToFolderRowMap;
@@ -67,19 +60,19 @@ namespace FaxListManagerCB {
     gboolean drag_motion(GtkWidget*, GdkDragContext*, gint, gint, guint, void*);
     gboolean folder_tree_view_motion_notify(GtkWidget*, GdkEventMotion*, void*);
     gboolean fax_tree_view_motion_notify(GtkWidget*, GdkEventMotion*, void*);
+    gboolean fax_tree_view_mouse_click(GtkWidget*, GdkEventButton*, void*);
     void drag_data_get(GtkWidget*, GdkDragContext*, GtkSelectionData*,
 		       guint, guint, void*);
     void drag_data_received(GtkWidget*, GdkDragContext*, gint, gint,
 			    GtkSelectionData*, guint, guint, void*);
     void fax_selected(GtkTreeSelection*, void*);
     void folder_selected(GtkTreeSelection*, void*);
-    gboolean write_path_timer_event(void*);
     void toggle_sort_type(GtkTreeViewColumn*, void*);
   } // extern "C"
 }   // namespace FaxListManagerCB
 }   // anonymous namespace
 
-class FaxListManager: public sigc::trackable {
+class FaxListManager {
 
   static bool fax_received_list_main_iteration;
   static bool fax_sent_list_main_iteration;
@@ -127,6 +120,7 @@ class FaxListManager: public sigc::trackable {
   void write_path(void);
   void write_paths_for_level(const GtkTreeIter*, std::ofstream&);
   std::string convert_faxname_to_date(const std::string&);
+  void write_path_timer_cb(bool&) {write_path();}
 
 public:
   friend void FaxListManagerCB::folder_tree_view_drag_begin(GtkWidget*, GdkDragContext*, void*);
@@ -134,16 +128,17 @@ public:
   friend gboolean FaxListManagerCB::drag_motion(GtkWidget*, GdkDragContext*, gint, gint, guint, void*);
   friend gboolean FaxListManagerCB::folder_tree_view_motion_notify(GtkWidget*, GdkEventMotion*, void*);
   friend gboolean FaxListManagerCB::fax_tree_view_motion_notify(GtkWidget*, GdkEventMotion*, void*);
+  friend gboolean FaxListManagerCB::fax_tree_view_mouse_click(GtkWidget*, GdkEventButton*, void*);
   friend void FaxListManagerCB::drag_data_get(GtkWidget*, GdkDragContext*, GtkSelectionData*,
 					      guint, guint, void*);
   friend void FaxListManagerCB::drag_data_received(GtkWidget*, GdkDragContext*, gint, gint,
 						   GtkSelectionData*, guint, guint, void*);
   friend void FaxListManagerCB::fax_selected(GtkTreeSelection*, void*);
   friend void FaxListManagerCB::folder_selected(GtkTreeSelection*, void*);
-  friend gboolean FaxListManagerCB::write_path_timer_event(void*);
   friend void FaxListManagerCB::toggle_sort_type(GtkTreeViewColumn*, void*);
 
-  sigc::signal0<void> selection_notify;
+  Emitter selection_notify;
+  Emitter fax_double_click_notify;
 
   void insert_folder_tree_view(GtkContainer* container_p) {
     gtk_container_add(container_p, GTK_WIDGET(folder_tree_view_p));
