@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2005 Chris Vine
+/* Copyright (C) 2001 to 2005 and 2009 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -12,40 +12,23 @@ COPYING distributed with the source files.
 
 #include <cstdlib>
 #include <vector>
+#include <algorithm>
 #include <cstring>
 
-#include <gtk/gtkcontainer.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkbox.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkbbox.h>
-#include <gtk/gtkhbbox.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtktoolbar.h>
-#include <gtk/gtktooltips.h>
-#include <gtk/gtkiconfactory.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtkliststore.h>
-#include <gtk/gtkcellrenderer.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtktreeviewcolumn.h>
-#include <gtk/gtkversion.h>
-
-#include <gdk/gdkpixbuf.h>
+#include <gdk/gdk.h>
 
 #include "file_list.h"
 #include "dialogs.h"
 #include "file_list_icons.h"
-#include "utils/mutex.h"
+
 #include "utils/tree_path_handle.h"
-#include "utils/shared_handle.h"
-#include "utils/utf8_utils.h"
 #include "utils/toolbar_append_widget.h"
 
+#include <c++-gtk-utils/shared_handle.h>
+#include <c++-gtk-utils/mutex.h>
+#include <c++-gtk-utils/mem_fun.h>
+#include <c++-gtk-utils/convert.h>
+
 #ifdef ENABLE_NLS
 #include <libintl.h>
 #endif
@@ -77,7 +60,7 @@ void FileListDialogCB::file_list_button_clicked(GtkWidget* widget_p, void* data)
     instance_p->move_down();
   }
   else if (widget_p == instance_p->add_button_p) {
-    instance_p->add_file();
+    instance_p->add_files();
   }
   else if (widget_p == instance_p->view_button_p) {
     instance_p->view_file();
@@ -140,7 +123,7 @@ FileListDialog::FileListDialog(const int standard_size_, GtkWindow* parent_p):
   GtkToolbar* toolbar_p = (GTK_TOOLBAR(gtk_toolbar_new()));
 
   gtk_button_box_set_layout(GTK_BUTTON_BOX(button_box_p), GTK_BUTTONBOX_END);
-  gtk_button_box_set_spacing(GTK_BUTTON_BOX(button_box_p), standard_size/2);
+  gtk_box_set_spacing(GTK_BOX(button_box_p), standard_size/2);
   
   gtk_scrolled_window_set_shadow_type(file_list_scrolled_window_p, GTK_SHADOW_IN);
   gtk_scrolled_window_set_policy(file_list_scrolled_window_p,
@@ -172,14 +155,12 @@ FileListDialog::FileListDialog(const int standard_size_, GtkWindow* parent_p):
   gtk_tree_view_set_reorderable(tree_view_p, true);
 
   // set up the tool bar
+#if GTK_CHECK_VERSION(2,16,0)
+  gtk_orientable_set_orientation(GTK_ORIENTABLE(toolbar_p), GTK_ORIENTATION_HORIZONTAL);
+#else
   gtk_toolbar_set_orientation(toolbar_p, GTK_ORIENTATION_HORIZONTAL);
+#endif
   gtk_toolbar_set_style(toolbar_p, GTK_TOOLBAR_ICONS);
-  gtk_toolbar_set_tooltips(toolbar_p, true);
-
-  // this is OK as tooltips is a public member of GtkToolbar
-  GtkTooltips* tooltips_p = toolbar_p->tooltips;
-  // is this necessary after calling gtk_toolbar_set_tooltips()?
-  gtk_tooltips_enable(tooltips_p);
 
   GtkWidget* image_p;
   { // scope block for the GobjHandle
@@ -189,7 +170,7 @@ FileListDialog::FileListDialog(const int standard_size_, GtkWindow* parent_p):
     gtk_container_add(GTK_CONTAINER(view_button_p), image_p);
   }
   toolbar_append_widget(toolbar_p, view_button_p,
-			gettext("View selected file"), "");
+			gettext("View selected file"));
   gtk_widget_set_sensitive(view_button_p, false);
 
   { // scope block for the GobjHandle
@@ -199,7 +180,7 @@ FileListDialog::FileListDialog(const int standard_size_, GtkWindow* parent_p):
     gtk_container_add(GTK_CONTAINER(remove_button_p), image_p);
   }
   toolbar_append_widget(toolbar_p, remove_button_p,
-			gettext("Remove selected file from list"), "");
+			gettext("Remove selected file from list"));
   gtk_widget_set_sensitive(remove_button_p, false);
 
   GtkWidget* add_hbox_p = gtk_hbox_new(false, 0);
@@ -213,7 +194,7 @@ FileListDialog::FileListDialog(const int standard_size_, GtkWindow* parent_p):
     gtk_container_add(GTK_CONTAINER(add_button_p), add_hbox_p);
   }
   toolbar_append_widget(toolbar_p, add_button_p,
-			gettext("Add files to the file list for faxing"), "");
+			gettext("Add files to the file list for faxing"));
 
   // bring up the icon size registered in MainWindow::MainWindow()
   GtkIconSize efax_gtk_button_size = gtk_icon_size_from_name("EFAX_GTK_BUTTON_SIZE");
@@ -224,8 +205,8 @@ FileListDialog::FileListDialog(const int standard_size_, GtkWindow* parent_p):
   image_p = gtk_image_new_from_stock(GTK_STOCK_GO_DOWN, efax_gtk_button_size);
   gtk_container_add(GTK_CONTAINER(down_button_p), image_p);
 
-  gtk_tooltips_set_tip(tooltips_p, up_button_p, gettext("Move file up"), "");
-  gtk_tooltips_set_tip(tooltips_p, down_button_p, gettext("Move file down"), "");
+  gtk_widget_set_tooltip_text(up_button_p, gettext("Move file up"));
+  gtk_widget_set_tooltip_text(down_button_p, gettext("Move file down"));
 
   gtk_container_add(GTK_CONTAINER(button_box_p), cancel_button_p);
   gtk_container_add(GTK_CONTAINER(button_box_p), ok_button_p);
@@ -287,25 +268,32 @@ FileListDialog::FileListDialog(const int standard_size_, GtkWindow* parent_p):
   g_signal_connect(G_OBJECT(selection_p), "changed",
 		   G_CALLBACK(FileListDialogCB::file_list_set_buttons), this);
 
+#if GTK_CHECK_VERSION(2,20,0)
+  gtk_widget_set_can_default(ok_button_p, true);
+  gtk_widget_set_can_default(cancel_button_p, true);
+  gtk_widget_set_can_focus(up_button_p, false);
+  gtk_widget_set_can_focus(down_button_p, false);
+#else
   GTK_WIDGET_SET_FLAGS(ok_button_p, GTK_CAN_DEFAULT);
   GTK_WIDGET_SET_FLAGS(cancel_button_p, GTK_CAN_DEFAULT);
   GTK_WIDGET_UNSET_FLAGS(up_button_p, GTK_CAN_FOCUS);
   GTK_WIDGET_UNSET_FLAGS(down_button_p, GTK_CAN_FOCUS);
+#endif
 
   gtk_container_set_border_width(GTK_CONTAINER(table_p), standard_size/3);
   gtk_container_add(GTK_CONTAINER(get_win()), file_list_box_p);
   gtk_window_set_type_hint(get_win(), GDK_WINDOW_TYPE_HINT_DIALOG);
   gtk_window_set_position(get_win(), GTK_WIN_POS_CENTER_ON_PARENT);
   gtk_window_set_default_size(get_win(), standard_size * 15, standard_size * 12);
-#if GTK_CHECK_VERSION(2,4,0)
+
   gtk_toolbar_set_show_arrow(GTK_TOOLBAR(toolbar_p), false);
-#endif
+
   gtk_widget_show_all(GTK_WIDGET(get_win()));
 
   gtk_button_set_relief(GTK_BUTTON(add_button_p), GTK_RELIEF_NORMAL);
   gtk_widget_set_sensitive(add_button_p, true);
   
-  // not needed if gtk_toolbar_set_show_arrow() has been set false, or GTK+ version < 2.4
+  // not needed if gtk_toolbar_set_show_arrow() has been set false
   /*
   gtk_widget_set_size_request(GTK_WIDGET(toolbar_p), 
 			      add_button_p->allocation.width + view_button_p->allocation.width
@@ -431,24 +419,26 @@ void FileListDialog::move_down(void) {
   else beep();
 }
 
-void FileListDialog::add_file(void) {
+void FileListDialog::add_files(void) {
 
   FileReadSelectDialog file_dialog(standard_size, true, get_win());
   file_dialog.exec();
   std::vector<std::string> file_result = file_dialog.get_result();
   if (!file_result.empty()) {
-    std::vector<std::string>::const_iterator str_iter;
-    for (str_iter = file_result.begin(); str_iter != file_result.end(); ++str_iter) {
-      
-      GtkTreeIter row_iter;
-      gtk_list_store_append(GTK_LIST_STORE(list_store_h.get()), &row_iter);
-      gtk_list_store_set(GTK_LIST_STORE(list_store_h.get()), &row_iter,
-			 ModelColumns::file_name, str_iter->c_str(),
-			 -1);
-    }
+    std::for_each(file_result.begin(), file_result.end(),
+		  MemFun::make(*this, &FileListDialog::add_file_item));
   }
 }
 
+void FileListDialog::add_file_item(const std::string& item) {
+
+  GtkTreeIter row_iter;
+  gtk_list_store_append(GTK_LIST_STORE(list_store_h.get()), &row_iter);
+  gtk_list_store_set(GTK_LIST_STORE(list_store_h.get()), &row_iter,
+		     ModelColumns::file_name, item.c_str(),
+		     -1);
+}
+
 std::pair<const char*, char* const*> FileListDialog::get_view_file_parms(const std::string& file_name) {
 
   std::vector<std::string> view_parms;
@@ -612,7 +602,7 @@ void FileListDialog::remove_file_prompt(void) {
     
     PromptDialog* dialog_p = new PromptDialog(msg.c_str(), gettext("Remove file"),
 					      standard_size, get_win());
-    dialog_p->accepted.connect(sigc::mem_fun(*this, &FileListDialog::remove_file));
+    dialog_p->accepted.connect(Callback::make(*this, &FileListDialog::remove_file));
     // there is no memory leak -- the memory will be deleted when PromptDialog closes
   }
 }
