@@ -6,20 +6,9 @@ COPYING distributed with the source files.
 
 */
 
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkbox.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkbbox.h>
-#include <gtk/gtkhbbox.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtktextview.h>
-#include <gtk/gtktextbuffer.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtklabel.h>
-
 #include <pango/pango-font.h>
-#include <gdk/gdkpixbuf.h>
-#include <gdk/gdktypes.h>
+
+#include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h> // the key codes are here
 
 #include "helpfile.h"
@@ -77,6 +66,13 @@ HelpDialog::HelpDialog(const int standard_size):
   label_p = gtk_label_new(gettext("Sending"));
   gtk_notebook_append_page(notebook_p, scrolled_window_p, label_p);
 
+  // make and append redialling help page
+  scrolled_window_p = make_scrolled_window();
+  gtk_container_add(GTK_CONTAINER(scrolled_window_p),
+		    make_text_view(get_redial_help()));
+  label_p = gtk_label_new(gettext("Auto redialing"));
+  gtk_notebook_append_page(notebook_p, scrolled_window_p, label_p);
+
   // make and append receiving help page
   scrolled_window_p = make_scrolled_window();
   gtk_container_add(GTK_CONTAINER(scrolled_window_p),
@@ -126,7 +122,11 @@ HelpDialog::HelpDialog(const int standard_size):
   
   gtk_widget_grab_focus(GTK_WIDGET(get_win()));
 
+#if GTK_CHECK_VERSION(2,20,0)
+  gtk_widget_set_can_default(close_button_p, true);
+#else
   GTK_WIDGET_SET_FLAGS(close_button_p, GTK_CAN_DEFAULT);
+#endif
 
   gtk_widget_show_all(GTK_WIDGET(get_win()));
 }
@@ -148,7 +148,12 @@ GtkWidget* HelpDialog::make_text_view(const char* text) {
   gtk_text_buffer_set_text(gtk_text_view_get_buffer(text_view_p),
 			   text,
 			   -1);
+#if GTK_CHECK_VERSION(2,20,0)
+  gtk_widget_set_can_focus(GTK_WIDGET(text_view_p), false);
+#else
   GTK_WIDGET_UNSET_FLAGS(GTK_WIDGET(text_view_p), GTK_CAN_FOCUS);
+#endif
+
   return GTK_WIDGET(text_view_p);
 }
 
@@ -170,14 +175,14 @@ const char* HelpDialog::get_sending_help(void) {
 		 "-------------\n"
 		 "\n"
 		 "Before sending a fax, it must be specified in the \"Fax to send\" box. "
-		 "It must be in postscript format (a format produced by all Unix/Linux "
+		 "It must be in postscript or PDF format (formats produced by all Unix/Linux "
 		 "word and document processors), and will be converted by the program "
 		 "into the correct tiffg3 fax format.\n"
 		 "\n"
 		 "There are two fax entry methods.  First, the fax to be sent can be a "
 		 "file saved on the filesystem.  It can be entered manually in the \"Fax "
 		 "to send\" box, or entered by means of the file selection dialog.  If "
-		 "the file comprises a single postscript file, then you can find it by "
+		 "the file comprises a single postscript or PDF file, then you can find it by "
 		 "pressing the \"Single File\" button.  It can be more easily found with "
 		 "this dialog if it is placed in the $HOME/faxout directory.\n"
 		 "\n"
@@ -229,6 +234,47 @@ const char* HelpDialog::get_sending_help(void) {
 		 "\"Using the fax lists\" further below.\n");
 }
 
+const char* HelpDialog::get_redial_help(void) {
+
+  return gettext("\n"
+		 "Automatic redialing\n"
+		 "-------------------\n"
+		 "\n"
+		 "The program settings have an option for automatic redialing of a fax "
+		 "to be sent if the modem was in use or the receiving station was busy "
+		 "when the first attempt to send it was made.  A list of the faxes "
+		 "queued for redialing can be brought up from the 'File' pull-down menu "
+		 "or from the icon in the system tray.\n"
+		 "\n"
+		 "If the program settings have been set to provide for automatic "
+		 "redialing, the program will keep trying to send the fax at the "
+		 "intervals chosen in the settings until either (a) the sending of the "
+		 "fax is successful, (b) there has been a failure arising from something "
+		 "other than the modem being in use or the recipient station being busy "
+		 "(such as a modem error, or the remote number ringing out without "
+		 "answering), (c) the fax is removed by the user from the list of faxes "
+		 "queued for redialing, (d) if the fax to be sent is a print job "
+		 "received from the print system, the print job is removed by the user "
+		 "from the dialog showing the list of queued faxes from the print "
+		 "socket, (e) if the fax to be sent comprises file(s) on the file "
+		 "system, the file(s) are deleted or moved (or their permissions changed "
+		 "to make them unreadable), or (f) the 'Stop' button is pressed while an "
+		 "attempt is being made to send the fax concerned which is queued for "
+		 "redialing.\n"
+		 "\n"
+		 "A fax queued for redialing will not be sent while the Settings dialog "
+		 "is open.  Instead, if the interval for resending such a fax has "
+		 "passed, a further sending attempt will take place once the Settings "
+		 "dialog has been closed.\n"
+		 "\n"
+		 "The list of queued faxes for redialing is not preserved when the "
+		 "program is terminated.  However, if any unsent faxes are print jobs "
+		 "received from the print system, they can still be found in the dialog "
+		 "showing the list of queued faxes from the print socket (assuming they "
+		 "have not been explicitly removed from that list by the user).\n");
+}
+
+
 const char* HelpDialog::get_receiving_help(void) {
 
   return gettext("\n"
