@@ -11,11 +11,10 @@ COPYING distributed with the source files.
 
 #include "prog_defs.h"
 
-#include <gtk/gtkwidget.h>
-#include <gtk/gtkwindow.h>
-#include <gtk/gtknotebook.h>
+#include <gtk/gtk.h>
+
+#include <c++-gtk-utils/window.h>
 
-#include "utils/window.h"
 
 namespace { // we put the functions in anonymous namespace in helpfile.cpp
             // so they are not exported at link time
@@ -32,6 +31,7 @@ class HelpDialog: public WinBase {
   GtkNotebook* notebook_p;
 
   const char* get_sending_help(void);
+  const char* get_redial_help(void);
   const char* get_receiving_help(void);
   const char* get_addressbook_help(void);
   const char* get_fax_list_help(void);
