@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2007 Chris Vine
+/* Copyright (C) 2001 to 2010 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -8,14 +8,6 @@ COPYING distributed with the source files.
 
 #include "prog_defs.h"
 
-// something in gtkprintjob.h clashes with the C++ headers with
-// libstdc++-5 and it needs to be included here to obtain a
-// clean compile
-#include <gtk/gtkversion.h>
-#if GTK_CHECK_VERSION(2,10,0)
-#include <gtk/gtkprintjob.h>
-#endif
-
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -34,69 +26,39 @@ COPYING distributed with the source files.
 #include <vector>
 #include <algorithm>
 #include <exception>
+#include <ios>
+#include <ostream>
+#include <istream>
+#include <sstream>
 #include <cstring>
 #include <ctime>
 
-#ifdef HAVE_OSTREAM
-#include <ios>
-#include <ostream>
-#endif
+#include <gtk/gtk.h>
+#include <gdk/gdk.h>
+#include <glib.h>
 
-#ifdef HAVE_ISTREAM
-#include <ios>
-#include <istream>
-#endif
-
-#ifdef HAVE_STRINGSTREAM
-#include <sstream>
-#else
-#include <strstream>
-#endif
+#include "mainwindow.h"
+#include "dialogs.h"
+#include "window_icon.h"
+#include "utils/icon_info_handle.h"
+#include "utils/utf8_utils.h"
 
 #ifdef HAVE_STREAM_IMBUE
 #include <locale>
 #endif
 
-#include <gtk/gtkmain.h>
-
-#include <gdk/gdkpixbuf.h>
-#include <gdk/gdk.h>   // for gdk_notify_startup_complete() and gdk_beep()
-
-#include <glib/gutils.h>
-#include <glib/gthread.h>
-
 #ifdef HAVE_X11_XLIB_H
 #include <X11/Xlib.h>
 #include <gdk/gdkx.h>  // for GDK_DISPLAY()
 #endif
 
-#if GTK_CHECK_VERSION(2,4,0)
-#  define EFAX_GTK_USE_ICON_THEME 1
-#else
-#  undef EFAX_GTK_USE_ICON_THEME
-#endif
-
-#if GTK_CHECK_VERSION(2,2,0)
-#  define HAVE_NOTIFY_STARTUP 1
-#else
-#  undef HAVE_NOTIFY_STARTUP
-#endif
-
-#ifdef EFAX_GTK_USE_ICON_THEME
-#include <gtk/gtkicontheme.h>
-#include "utils/icon_info_handle.h"
-#endif
-
-#include "mainwindow.h"
-#include "dialogs.h"
-#include "window_icon.h"
-#include "utils/mutex.h"
-#include "utils/utf8_utils.h"
-#include "utils/gobj_handle.h"
-#include "utils/fdstream.h"
-#include "utils/pipes.h"
-#include "utils/notifier.h"
-#include "utils/gerror_handle.h"
+#include <c++-gtk-utils/mutex.h>
+#include <c++-gtk-utils/gobj_handle.h>
+#include <c++-gtk-utils/pipes.h>
+#include <c++-gtk-utils/notifier.h>
+#include <c++-gtk-utils/gerror_handle.h>
+#include <c++-gtk-utils/convert.h>
+#include <c++-gtk-utils/prog_present.h>
 
 #ifdef ENABLE_NLS
 #include <locale.h>
@@ -107,16 +69,22 @@ COPYING distributed with the source files.
 #define EXIT_SUCCESS 0
 #endif
 
-#define LOCKFILE_NAME "/.efax-gtk-pid"
-
 // the headers on one of my machines does not properly include the declaration
 // of getpgid() for some reason, so declare it here explicitly
 extern "C" pid_t getpgid(pid_t);
 
 Prog_config prog_config;
 
-bool is_program_running(void);
+void get_fonts(void);
+void get_window_icon(void);
+int mkdir_with_parent(const std::string&);
+bool is_arg(const char*, int, char*[]);
+void display_dbus_error_dialog(void);
+void clean_up_receive_dir(void);
+bool is_ascii(const std::string&);
 bool get_prog_parm(const char*, std::string&, std::string&, std::string(*)(const std::string&));
+extern "C" void atexit_cleanup(void);
+extern "C" gboolean present_prog(void*, const char**);
 
 inline bool get_prog_parm(const char* name, std::string& line, std::string& result) {
   return get_prog_parm(name, line, result, Utf8::locale_to_utf8);
@@ -127,14 +95,8 @@ inline bool not_ascii_char(char c) {
   return false;
 }
 
-void get_fonts(void);
-void get_window_icon(void);
-int mkdir_with_parent(const std::string&);
-bool is_arg(const char*, int, char*[]);
-extern "C" void atexit_cleanup(void);
-void clean_up_receive_dir(void);
-bool is_ascii(const std::string&);
-int is_in_process_table(pid_t);
+
+static MainWindow* main_win_p = 0;
 
 
 int main(int argc, char* argv[]) {
@@ -170,7 +132,7 @@ int main(int argc, char* argv[]) {
   if (argc > 1) {
     if (is_arg("--version", argc, argv)) {
       std::string message("efax-gtk-" VERSION "\n");
-      message += gettext("Copyright (C) 2001 - 2007 Chris Vine\n"
+      message += gettext("Copyright (C) 2001 - 2010 Chris Vine\n"
 			 "This program is released under the "
 			 "GNU General Public License, version 2\n");
       try {
@@ -209,13 +171,15 @@ int main(int argc, char* argv[]) {
   // (we do not use the GTK global thread lock, so there is no need to
   // call gdk_threads_init())
   g_thread_init(0);
-  prog_config.mutex_p = new Thread::Mutex;
-  
-  std::string messages(configure_prog(false));
+  gtk_init(&argc, &argv);
 
-  if(!is_program_running()) {
+  if (register_prog("efax_gtk",
+                    present_prog,
+                    0)) {
+
+    prog_config.mutex_p = new Thread::Mutex;
+    std::string messages(configure_prog(false));
 
-    gtk_init(&argc, &argv);
     g_atexit(atexit_cleanup);
 
     // initialise Notifier objects
@@ -223,7 +187,7 @@ int main(int argc, char* argv[]) {
       Notifier::init();
     }
     catch (PipeError& error) {
-      std::cerr << error.error_message;
+      std::cerr << error.what();
       std::exit(PIPE_ERROR);
     }
 
@@ -279,31 +243,30 @@ int main(int argc, char* argv[]) {
 
       try {
 	MainWindow mainwindow(messages, start_hidden, start_in_standby, filename);
+	main_win_p = &mainwindow;
 
 	// everything is set up
 	// now enter the main program loop
 	gtk_main();
       }
       catch (PipeError& error) {
-	std::cerr << error.error_message;
+	std::cerr << error.what();
 	std::exit(PIPE_ERROR);
       }
     }
+    main_win_p = 0;
+    delete prog_config.mutex_p;
   }
-#ifdef HAVE_NOTIFY_STARTUP
   else {
-    // if we have got here then is_program_running() will have detected
-    // that there is already an instance of this program running and
-    // it will have sent a message to that instance via a fifo to present
-    // itself on the current workspace.  We now need to tell the window
-    // manager that we have completed startup, or the window manager
-    // startup notifier will be confused
-    // gdk_notify_startup_complete() is first available with GTK+-2.2
-    gtk_init(&argc, &argv);
-    gdk_notify_startup_complete();
+    // for debugging
+    //const char* args[2] = {"Instance already exists - invoking via dbus\n", 0};
+    // normal use
+    const char** args = 0;
+    int ret = present_instance(args);
+    if (!ret) gdk_notify_startup_complete();
+    else display_dbus_error_dialog();
+    return ret;
   }
-#endif
-  delete prog_config.mutex_p;
   return 0;
 }
 
@@ -414,10 +377,12 @@ std::string configure_prog(bool reread) {
       prog_config.ps_view_cmd = "";
       prog_config.sock_server_port = "";
       prog_config.fax_received_prog = "";
+      prog_config.redial_interval = "";
       prog_config.logfile_name = "";
       prog_config.permitted_clients_list.clear();
     }
-    // if we are setting up for first time, initialise prog_config.efax_controller_child_pid;
+    // if we are setting up for first time, initialise prog_config.efax_controller_child_pid
+    // (we don't need a mutex here as no thread to send/receive a fax has yet been launched)
     else prog_config.efax_controller_child_pid = 0;
 
 // now extract settings from file
@@ -441,6 +406,7 @@ std::string configure_prog(bool reread) {
     std::string fax_received_popup;
     std::string fax_received_exec;
     std::string addr_in_header;
+    std::string redial;
     std::string work_subdir;
     
     while (std::getline(filein, file_read)) {
@@ -540,6 +506,12 @@ std::string configure_prog(bool reread) {
 	// look for "ADDR_IN_HEADER:"
 	else if (get_prog_parm("ADDR_IN_HEADER:", file_read, addr_in_header));
 
+	// look for "REDIAL:"
+	else if (get_prog_parm("REDIAL:", file_read, redial));
+
+	// look for "REDIAL_INTERVAL:"
+	else if (get_prog_parm("REDIAL_INTERVAL:", file_read, prog_config.redial_interval));
+
 	// look for "WORK_SUBDIR:"
 	else if (get_prog_parm("WORK_SUBDIR:", file_read, work_subdir,
 			       Utf8::filename_to_utf8));
@@ -700,7 +672,7 @@ std::string configure_prog(bool reread) {
       prog_config.resolution = "204x196";
     }
     else {
-      temp = Utf8::to_lower(prog_config.resolution);
+      temp = utf8_to_lower(prog_config.resolution);
       if (!temp.compare("fine")) prog_config.resolution = "204x196";
       else if (!temp.compare("standard")) prog_config.resolution = "204x98";
       else {
@@ -724,7 +696,7 @@ std::string configure_prog(bool reread) {
       prog_config.page_dim = "210x297mm";
     }
     else {
-      prog_config.page_size = Utf8::to_lower(prog_config.page_size);
+      prog_config.page_size = utf8_to_lower(prog_config.page_size);
       if (!prog_config.page_size.compare("a4")) prog_config.page_dim = "210x297mm";
       else if (!prog_config.page_size.compare("letter")) prog_config.page_dim = "216x279mm";
       else if (!prog_config.page_size.compare("legal")) prog_config.page_dim = "216x356mm";
@@ -738,7 +710,7 @@ std::string configure_prog(bool reread) {
     
     if (dialmode.empty()) prog_config.tone_dial = true;
     else {
-      temp = Utf8::to_lower(dialmode);
+      temp = utf8_to_lower(dialmode);
       if (!temp.compare("tone")) prog_config.tone_dial = true;
       else if (!temp.compare("pulse")) prog_config.tone_dial = false;
       else {
@@ -765,16 +737,14 @@ std::string configure_prog(bool reread) {
 	prog_config.parms.push_back(temp);
       }
     }
+
     if (prog_config.print_cmd.empty()) {
       prog_config.print_cmd = "lpr";
     }
     
-    temp = Utf8::to_lower(gtkprint);
-#if !(GTK_CHECK_VERSION(2,10,0))
-    temp = "no";
-#endif
-    if (!temp.compare("yes")) prog_config.gtkprint = true;
-    else prog_config.gtkprint = false;
+    temp = utf8_to_lower(gtkprint);
+    if (!temp.compare("no")) prog_config.gtkprint = false;
+    else prog_config.gtkprint = true;
 
     if (prog_config.print_shrink.empty()) {
       prog_config.print_shrink = "100";
@@ -784,7 +754,7 @@ std::string configure_prog(bool reread) {
       prog_config.print_shrink = "100";
     }
     
-    temp = Utf8::to_lower(print_popup);
+    temp = utf8_to_lower(print_popup);
     if (!temp.compare("no")) prog_config.print_popup = false;
     else prog_config.print_popup = true;
 
@@ -793,11 +763,11 @@ std::string configure_prog(bool reread) {
       prog_config.ps_view_cmd = "gv";
     }
 
-    temp = Utf8::to_lower(sock_server);
+    temp = utf8_to_lower(sock_server);
     if (!temp.compare("yes")) prog_config.sock_server = true;
     else prog_config.sock_server = false;
 
-    temp = Utf8::to_lower(sock_popup);
+    temp = utf8_to_lower(sock_popup);
     if (!temp.compare("yes")) prog_config.sock_popup = true;
     else prog_config.sock_popup = false;
 
@@ -815,7 +785,7 @@ std::string configure_prog(bool reread) {
       prog_config.sock_server_port = "";
     }
 
-    temp = Utf8::to_lower(sock_client_address);
+    temp = utf8_to_lower(sock_client_address);
     if (!temp.compare("other")) prog_config.other_sock_client_address = true;
     else prog_config.other_sock_client_address = false;
 
@@ -837,18 +807,33 @@ std::string configure_prog(bool reread) {
       }
     }
 
-    temp = Utf8::to_lower(fax_received_popup);
+    temp = utf8_to_lower(fax_received_popup);
     if (!temp.compare("yes")) prog_config.fax_received_popup = true;
     else prog_config.fax_received_popup = false;
 
-    temp = Utf8::to_lower(fax_received_exec);
+    temp = utf8_to_lower(fax_received_exec);
     if (!temp.compare("yes")) prog_config.fax_received_exec = true;
     else prog_config.fax_received_exec = false;
 
-    temp = Utf8::to_lower(addr_in_header);
+    temp = utf8_to_lower(addr_in_header);
     if (!temp.compare("no")) prog_config.addr_in_header = false;
     else prog_config.addr_in_header = true;
 
+    temp = utf8_to_lower(redial);
+    if (!temp.compare("yes")) prog_config.redial = true;
+    else prog_config.redial = false;
+
+    if (prog_config.redial_interval.empty()) {
+      prog_config.redial_interval = "15";
+    }
+
+    else if (std::atoi(prog_config.redial_interval.c_str()) < 5
+	     || std::atoi(prog_config.redial_interval.c_str()) > 1440) {
+      return_val += gettext("Invalid redial interval has been specified. "
+			    "It needs to be between 5 and 1440 minutes\n");
+      prog_config.redial_interval = "15";
+    }
+
     if (!reread) {  // prog_config.working_dir and prog_config.GPL_flag are not
                     // affected by the settings dialog and so not re-configurable
                     // after the program has initialised itself for the first time
@@ -955,8 +940,6 @@ void get_window_icon(void) {
 
   bool have_icon = false;
 
-#ifdef EFAX_GTK_USE_ICON_THEME
-
   GtkIconTheme* icon_theme_p = gtk_icon_theme_get_default();
 
   IconInfoScopedHandle icon_info_h(gtk_icon_theme_lookup_icon(icon_theme_p,
@@ -980,7 +963,6 @@ void get_window_icon(void) {
       }
     }
   }
-#endif
 
   if (!have_icon) {
     prog_config.window_icon_h =
@@ -988,6 +970,16 @@ void get_window_icon(void) {
   }
 }
 
+void display_dbus_error_dialog(void) {
+  InfoDialog dialog(gettext("DBUS error: Cannot start efax-gtk.\n\n"
+			    "Make sure the DBUS session message bus is running"),
+		    gettext("efax-gtk: DBUS error"),
+		    GTK_MESSAGE_ERROR,
+		    0,
+		    false);
+  dialog.exec();
+}
+
 int mkdir_with_parent(const std::string& dir) {
 
   // this function must be passed an absolute path name
@@ -1026,95 +1018,6 @@ bool is_arg(const char* arg, int argc, char* argv[]) {
   return return_val;
 }
 
-bool is_program_running(void) {
-
-  bool return_val = false;
-  int count = 0;
-
-  std::string lockfile(prog_config.working_dir);
-  lockfile += LOCKFILE_NAME;
-
-  // do an atomic exclusive write open on the lockfile
-  // we do not need to check for EINTR on the open as we do not catch
-  // any signals at this stage
-  int new_lock_fd = open(lockfile.c_str(), O_WRONLY | O_CREAT | O_EXCL, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
-  while (count < 2 && new_lock_fd == -1) {      // check to see why we could not open new_lock
-    if (access(lockfile.c_str(), F_OK) == -1) { // file doesn't exist, so there must be a problem with permissions
-
-      std::cerr << "Cannot create lockfile " << lockfile << std::endl;
-    }
-
-    else {   // the lockfile already exists - see if it is stale
-             // it shouldn't ever be stale with the other clean up provision
-             // in this program, but nonetheless ...
-      std::ifstream existing_lock(lockfile.c_str(), std::ios::in);
-      if(!existing_lock) { // can't open lock file
-	std::cerr << "Not able to access existing lockfile " << lockfile << std::endl;
-	// get rid of the old lockfile if we can
-	unlink(lockfile.c_str());
-      }
-      else { // lock file already exists and we can open it - check if stale
-
-#ifdef HAVE_STREAM_IMBUE
-	existing_lock.imbue(std::locale::classic());
-#endif // HAVE_STREAM_IMBUE
-
-	// test with kill() as well as with is_in_process_table()
-	// in case SUS 'ps' arguments are not available
-	pid_t existing_lock_pid;
-	existing_lock >> existing_lock_pid;
-	if (!is_in_process_table(existing_lock_pid)
-	    || (existing_lock && kill(existing_lock_pid, 0) < 0)) { // stale lock
-	  unlink(lockfile.c_str());    // delete stale lock file
-	  new_lock_fd = open(lockfile.c_str(), O_WRONLY | O_CREAT | O_EXCL,
-			     S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
-	  count++;                     // signal we can only have one more try if this open fails
-	}
-	else {                         // active lock - alert the program pipe
-	  std::cout << "Active lock - bringing up existing instance of efax-gtk" << std::endl;
-	  count = 2;                   // break out of the loop
-
-	  std::string fifo_name(prog_config.working_dir);
-	  fifo_name += PIPE_NAME;
-	  
-	  if (!access(fifo_name.c_str(), F_OK)) { // if we don't enter this block we have a
-                                                  // problem - the fifo should exist but doesn't
-	    int write_fd = open(fifo_name.c_str(), O_WRONLY | O_NONBLOCK);
-	    if (write_fd >= 0) {                  // if we don't enter this block we also have a
-                                                  // problem - the fifo can't be opened for writing
-	      Attachable::Out pipe_stream;
-
-#ifdef HAVE_STREAM_IMBUE
-	      pipe_stream.imbue(std::locale::classic());
-#endif // HAVE_STREAM_IMBUE
-
-	      pipe_stream.attach(write_fd);
-	      pipe_stream << existing_lock_pid << std::endl;
-	      pipe_stream.close();
-	      return_val = true;
-	    }
-	    else std::cerr << "Error: cannot open fifo for writing" << std::endl;
-	  }
-	  else std::cerr << "Error: program fifo doesn't exist" << std::endl;
-	}
-      }
-    }
-  }
-  if (count < 2 && new_lock_fd >= 0)  {// we have successfully created a lock file - now write our pid to it
-
-    Attachable::Out new_lock;
-
-#ifdef HAVE_STREAM_IMBUE
-    new_lock.imbue(std::locale::classic());
-#endif // HAVE_STREAM_IMBUE
-
-    new_lock.attach(new_lock_fd);
-    new_lock << std::setfill('0') << std::setw(10) << getpid() << std::endl;
-    new_lock.close();
-  }
-  return return_val;
-}
-
 bool is_ascii(const std::string& text) {
   
   std::string::const_iterator end_iter = text.end();
@@ -1124,86 +1027,6 @@ bool is_ascii(const std::string& text) {
   return false;
 }
 
-// returns 1 if efax-gtk is definitely in the process table for the PID in existing_lock_pid
-// returns 0 if efax-gtk is definitely not in process table for the PID in existing_lock_pid
-// returns -1 if we cannot tell because 'ps' fails (say because SUS ps arguments are not available)
-int is_in_process_table(pid_t existing_lock_pid) {
-
-  // call ourselves recursively to see whether we can find out own process id in the process table
-  static bool checked = false;
-  if (!checked) {
-    checked = true;
-    if (is_in_process_table(getpid()) != 1) {
-      const std::string message("'ps' command is not POSIX conforming or is not in PATH\n");
-      write(2, message.data(), message.size());
-      return -1;
-    }
-  }
-
-  PipeFifo fork_pipe(PipeFifo::block);
-  // now fork to create the process which send the output of 'ps' to stdout
-  pid_t pid = fork();
-
-  if (pid == -1) {
-    const std::string message("Fork error - exiting\n");
-    write(2, message.data(), message.size());
-    std::exit(FORK_ERROR);
-  }
-  
-  if (!pid) { // child process which will send the output of 'ps' to stdout
-  
-    // this is the child process to write to stdout
-    fork_pipe.connect_to_stdout();
-
-    // the program is not multi-threaded at this point so there is no problem
-    // with using non async-signal-safe functions before we exec()
-#ifdef HAVE_STRINGSTREAM
-    std::ostringstream strm;
-#  ifdef HAVE_STREAM_IMBUE
-    strm.imbue(std::locale::classic());
-#  endif
-    strm << "-p" << existing_lock_pid;
-    execlp("ps", "ps", strm.str().c_str(), static_cast<char*>(0));
-#else
-    std::ostrstream strm;
-#  ifdef HAVE_STREAM_IMBUE
-    strm.imbue(std::locale::classic());
-#  endif
-    strm << "-p" << existing_lock_pid << std::ends;
-    execlp("ps", "ps", strm.str(), static_cast<char*>(0));
-#endif
-
-    // if we reached this point, then the execlp() call must have failed
-    _exit(EXIT_SUCCESS - 1);
-  }
-
-  // read from the pipe
-  fork_pipe.make_readonly();
-  ssize_t result;
-  char buffer[PIPE_BUF];
-  std::string output;
-  while ((result = fork_pipe.read(buffer, PIPE_BUF)) > 0) {
-    output.append(buffer, result);
-  }
-
-  // for debugging
-  //write(1, output.data(), output.size());
-
-  // we can safely use wait() here because we are setting the program up
-  // and nothing else is calling wait() or waitpid() and we do not need
-  // to worry about EINTR because there are no signals at this stage
-  int status;
-  if (output.empty() || wait(&status) == -1) return -1;
-  int exit_code = EXIT_SUCCESS - 1;
-  if (WIFEXITED(status)) exit_code = WEXITSTATUS(status);
-  if (exit_code != EXIT_SUCCESS) return -1;
-
-  if (output.find("efax-gtk") != std::string::npos) {
-    return 1;
-  }
-  return 0;
-}
-
 void atexit_cleanup(void) {
 
   // this does any essential clean up, which is always required to make the program
@@ -1228,16 +1051,6 @@ void atexit_cleanup(void) {
   }
   else std::cerr << "Error: we are not a process group leader!" << std::endl;
 
-  // remove program lock file
-  std::string lockfile(prog_config.working_dir);
-  lockfile += LOCKFILE_NAME;
-  unlink(lockfile.c_str());
-
-  // remove program fifo
-  std::string fifo_name(prog_config.working_dir);
-  fifo_name += PIPE_NAME;
-  unlink(fifo_name.c_str());
-
   // clean up the receive_directory if necessary
   clean_up_receive_dir();
 
@@ -1384,3 +1197,11 @@ void clean_up_receive_dir(void) {
     }
   }
 }
+
+gboolean present_prog(void*, const char** args) {
+  if (main_win_p) main_win_p->present_window();
+
+  // just use args for debugging here
+  if (args) std::cout << args[0];
+  return TRUE;
+}
