@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2007 Chris Vine
+/* Copyright (C) 2001 to 2010 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -18,24 +18,10 @@ COPYING distributed with the source files.
 #include <vector>
 #include <list>
 #include <utility>
-#include <memory>
-
-#include <gtk/gtkwidget.h>
-#include <gtk/gtkwindow.h>
-#include <gtk/gtkstyle.h>
-#include <gtk/gtkmenuitem.h>
-#include <gtk/gtktexttag.h>
-// there is a problem with the GTK+ headers in some versions: before including gtktextmark.h
-// we need to include gtktextiter.h or there will be a compile error in main.cpp
-#include <gtk/gtktextiter.h>
-#include <gtk/gtktextmark.h>
-
-#include <gdk/gdkevents.h>
-#include <gdk/gdkgc.h>
-#include <glib/gtypes.h>
-
-#include <sigc++/sigc++.h>
 
+#include <gtk/gtk.h>
+#include <gdk/gdk.h>
+#include <glib.h>
 
 #include "efax_controller.h"
 #include "fax_list_manager.h"
@@ -45,44 +31,44 @@ COPYING distributed with the source files.
 #include "tray_icon.h"
 #include "helpfile.h"
 #include "logger.h"
-#include "utils/pipes.h"
-#include "utils/shared_ptr.h"
-#include "utils/mutex.h"
-#include "utils/widget.h"
-#include "utils/gobj_handle.h"
-#include "utils/notifier.h"
-#include "utils/io_watch.h"
-#include "utils/utf8_utils.h"
+
+#include <c++-gtk-utils/pipes.h>
+#include <c++-gtk-utils/shared_ptr.h>
+#include <c++-gtk-utils/mutex.h>
+#include <c++-gtk-utils/window.h>
+#include <c++-gtk-utils/widget.h>
+#include <c++-gtk-utils/gobj_handle.h>
+#include <c++-gtk-utils/reassembler.h>
+#include <c++-gtk-utils/emitter.h>
 
 #define MAINWIN_SAVE_FILE ".efax-gtk_mainwin_save"
 
 
-class MessageText: public sigc::trackable, public MainWidgetBase {
+class MessageText: public MainWidgetBase {
 
   GtkWidget* text_view_p;
   GtkTextMark* end_mark_p;
   GobjHandle<GtkTextTag> red_tag_h;
-  std::auto_ptr<Logger> logger_a;
-  GtkWindow* window_p;
+  Logger logger;
 
   void cleanify(std::string&);
 public:
-  void write_black_slot(const char* message);
-  void write_red_slot(const char* message);
-  void print_log(void) {logger_a->print_log();}
+  void write_black_cb(const char* message);
+  void write_red_cb(const char* message);
+  void print_log(void) {logger.print_log();}
   static void print_page_setup(GtkWindow* parent_p) {Logger::print_page_setup(parent_p);}
-  void view_log(const int standard_size) {logger_a->view_log(standard_size);}
-  void reset_logfile(void) {logger_a->reset_logfile();}
+  void view_log(const int standard_size) {logger.view_log(standard_size);}
+  void reset_logfile(void) {logger.reset_logfile();}
   MessageText(GtkWindow* window_p);
 };
 
-class StatusLine: public sigc::trackable, public MainWidgetBase {
+class StatusLine: public MainWidgetBase {
 
   const int standard_size;
   GtkWidget* status_label_p;
 public:
   void set_status_line(int);
-  void write_status_slot(const char* text);
+  void write_status_cb(const char* text);
   StatusLine(const int);
 };
 
@@ -98,12 +84,11 @@ namespace MainWindowCB {
     void mainwin_style_set(GtkWidget*, GtkStyle*, void*);
     gboolean mainwin_timer_event(void*);
     gboolean mainwin_drawing_area_expose_event(GtkWidget*, GdkEventExpose*, void*);
-    gboolean mainwin_start_hidden_check(void*);
   }
 }
 }
 
-class MainWindow: public sigc::trackable, public WinBase {
+class MainWindow: public WinBase {
 
   const int standard_size;
   static PipeFifo error_pipe;
@@ -114,11 +99,8 @@ class MainWindow: public sigc::trackable, public WinBase {
   std::string prog_fifo_name;
   guint error_pipe_tag;
 
-  sigc::connection socket_dialog_connection;
-  sigc::connection update_socket_list_connection;
-  sigc::signal1<void, std::pair<SharedPtr<FilenamesList>, SharedPtr<Thread::Mutex::Lock> > > update_socket_list;
-  sigc::signal0<void> close_socket_list_dialog;
-  Notifier present_window_notify;
+  Emitter close_socket_list_dialog;
+  EmitterArg<std::pair<SharedPtr<FilenamesList>, SharedPtr<Thread::Mutex::Lock> > > update_socket_list;
 
   GtkWidget* drawing_area_p;
   GtkWidget* file_button_p;
@@ -141,6 +123,7 @@ class MainWindow: public sigc::trackable, public WinBase {
   GtkMenuItem* single_file_item_p;
   GtkMenuItem* multiple_file_item_p;
   GtkMenuItem* address_book_item_p;
+  GtkMenuItem* redial_queue_item_p;
   GtkMenuItem* settings_item_p;
   GtkMenuItem* quit_item_p;
   GtkMenuItem* print_log_item_p;
@@ -165,7 +148,7 @@ class MainWindow: public sigc::trackable, public WinBase {
   SocketListDialog* socket_list_p;
   HelpDialog* helpfile_p;
 
-  TrayItem tray_item;
+  TrayIcon tray_icon;
 
   Utf8::Reassembler error_pipe_reassembler;
 
@@ -177,6 +160,7 @@ class MainWindow: public sigc::trackable, public WinBase {
   void file_list_impl(void);
   void socket_list_impl(void);
   void addressbook_impl(void);
+  void redial_queue_impl(void);
   void receive_impl(EfaxController::State);
   void fax_list_impl(FaxListEnum::Mode);
   void settings_impl(void);
@@ -187,23 +171,24 @@ class MainWindow: public sigc::trackable, public WinBase {
   void set_socket_items_sensitive_impl(void);
   void style_set_impl(void);
 
-  // these are connected to sigc signals or Notifier objects, or connected
+  // these are connected to Emitter or Notifier objects, or connected
   // to the callback of a WatchSource (iowatch) object, and might also be
   // called by the handlers of GTK+ signals
-  void sendfax_slot(void);
-  void set_files_slot(const std::string&);
-  void fax_received_notify_slot(const std::pair<std::string, std::string>&);
-  void enter_socket_file_slot(const std::pair<std::string, std::string>&);
-  void settings_changed_slot(const std::string&);
-  void socket_filelist_changed_notify_slot(void);
-  void socket_filelist_closed_slot(void);
-  void fax_to_send_notify_slot(void);
-  void set_number_slot(const std::string&);
-  void tray_icon_menu_slot(int);
-  void tray_icon_left_clicked_slot(void);
-  void quit_slot(void);
-  void present_window_slot();
-  bool read_error_pipe_slot(void);
+  void sendfax_cb(void);
+  void set_files_cb(const std::string&);
+  void fax_received_notify_cb(const std::pair<std::string, std::string>&);
+  void enter_socket_file_cb(const std::pair<std::string, std::string>&);
+  void settings_changed_cb(const std::string&);
+  void socket_filelist_changed_notify_cb(void);
+  void fax_to_send_notify_cb(void);
+  void nullify_notified_fax_cb(void);
+  void set_number_cb(const std::string&);
+  void tray_icon_menu_cb(int);
+  void tray_icon_activated_cb(void);
+  void quit_cb(void);
+  void read_error_pipe_cb(bool&);
+  void sig_event_cb(bool&);
+  void start_hidden_check_cb(bool&);
 
   void fax_to_send_dialog(const std::pair<std::string, unsigned int>&);
   void get_longest_button_text(void);
@@ -211,8 +196,6 @@ class MainWindow: public sigc::trackable, public WinBase {
   void get_window_settings(void);
   void save_window_settings(void);
   void strip(std::string&);
-  void start_pipe_thread(void);
-  void pipe_thread(void);
 protected:
   virtual void on_delete_event(void);
 public:
@@ -224,15 +207,14 @@ public:
   friend gboolean MainWindowCB::mainwin_window_state_event(GtkWidget*,
 							   GdkEventWindowState*, void*);
   friend void MainWindowCB::mainwin_style_set(GtkWidget*, GtkStyle*, void*);
-  friend gboolean MainWindowCB::mainwin_timer_event(void*);
   friend gboolean MainWindowCB::mainwin_drawing_area_expose_event(GtkWidget*,
 								  GdkEventExpose*, void*);
-  friend gboolean MainWindowCB::mainwin_start_hidden_check(void*);
 
   friend ssize_t write_error(const char*);
   friend int connect_to_stderr(void);
 
   void remove_from_socket_server_filelist(const std::string& file);
+  void present_window(void);
   MainWindow(const std::string&, bool start_hidden = false,
 	     bool start_in_standby = false, const char* filename = 0);
   ~MainWindow(void);
