@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2005 Chris Vine
+/* Copyright (C) 2001 to 2010 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -20,10 +20,6 @@ inline const char* gettext(const char* text) {
 }
 #endif
 
-#ifndef HAVE_SSIZE_T
-typedef signed int ssize_t;
-#endif
-
 #ifndef HAVE_SOCKLEN_T
 typedef unsigned int socklen_t;
 #endif
@@ -32,11 +28,6 @@ typedef unsigned int socklen_t;
 typedef unsigned long in_addr_t;
 #endif
 
-// deal with libsigc++
-#if SIGC_VERSION < 20
-#include "sigc_compatibility.h"
-#endif
-
 // define some common exit codes
 
 #define MEM_ERROR 10
@@ -54,18 +45,13 @@ typedef unsigned long in_addr_t;
 #include <sys/types.h>
 #include <vector>
 #include <string>
-#include <gdk/gdkpixbuf.h>
-#include "utils/gobj_handle.h"
+#include <gdk/gdk.h>
 
-// forward declaration for Prog_config
-namespace Thread {
-  class Mutex;
-}
+#include <c++-gtk-utils/gobj_handle.h>
+#include <c++-gtk-utils/mutex.h>
 
-
-using namespace std;    // until I get a Std C++ compliant compiler to test with
-
-#define PIPE_NAME "/.efax-gtk-pipe"
+namespace Cgu {}
+using namespace Cgu;
 
 /************************* threading issues *************************
 
@@ -106,34 +92,40 @@ using namespace std;    // until I get a Std C++ compliant compiler to test with
        main GUI thread and by the socket_server_thread - see the
        comments in that method about mutexes)
 
-   - a temporary thread created to convert a postscript file to tiffg3
-     files for faxing, involving the following methods of the
-     EfaxController class:
+   - a temporary thread created to send a fax, involving the following
+     methods of the EfaxController class:
 
-       make_fax_thread() and get_gs_parms()
+       sendfax_thread(), ps_file_to_tiff_files(), get_gs_parms(),
+       get_efax_parms(), delete_parms() and join_child()
+
+   - a temporary thread created to receive a fax, involving the
+     following methods of the EfaxController class:
+
+       receive_thread(), get_efax_parms(), delete_parms() and
+       join_child()
 
    - a temporary thread to prepare a fax for printing, involving the
      following methods of the FaxListDialog class:
 
-       print_fax_thread(), write_pipe_to_file(),
-       get_print_from_stdin_parms(), get_fax_to_ps_parms() and
-       delete_parms() (write_pipe_to_file() and get_fax_to_ps_parms()
+       print_fax_thread(), get_print_from_stdin_parms(),
+       get_fax_to_ps_parms() and delete_parms() (get_fax_to_ps_parms()
        is also called by the temporary thread invoked with
        view_fax_thread() and delete_parms() is also called by that
        thread and by the main (GUI) thread)
 
-     together with FilePrintManager::set_filename() and
-     FilePrintManager::print() if the program is compiled against
-     GTK+-2.10 or higher.
+     together with the following methods of the MonoTiffPrintManager
+     class:
+
+       set_filenames() and print()
 
    - a temporary thread to view a fax, involving the following methods
      of the FaxListDialog class:
 
        view_fax_thread(), write_pipe_to_file(), get_ps_viewer_parms(),
-       get_fax_to_ps_parms() and delete_parms() (write_pipe_to_file()
-       and get_fax_to_ps_parms() is also called by the temporary
-       thread invoked with print_fax_thread() and delete_parms() is
-       also called by that thread and by the main (GUI) thread)
+       get_fax_to_ps_parms() and delete_parms() (get_fax_to_ps_parms()
+       is also called by the temporary thread invoked with
+       print_fax_thread() and delete_parms() is also called by that
+       thread and by the main (GUI) thread)
 
 3. The last member of the Prog_config class below is a pointer to a mutex object
    allocated in int main() in main.cpp.  The mutex should be locked whenever:
@@ -192,19 +184,49 @@ using namespace std;    // until I get a Std C++ compliant compiler to test with
 5. See also the comments about thread sefety in the function
    write_error() in mainwindow.cpp.
 
-6. It would have been convenient to have signal handling in a separate
-   thread and then to sigwait() on the signals used by this program.
-   However the main signal of interest is SIGCHLD, from which the
-   termination of a send or receive session from efax can be detected,
-   and information about the success or failure of the sending or
-   receipt of the fax can be obtained and acted on.  Unfortunately,
-   linuxthreads does not properly implement POSIX threads as regards
-   signal handling, and with linuxthreads only the thread which
-   fork()s will receive SIGCHLD.  Thus SIGCHLD cannot be received in a
-   special signal handling thread.  Instead, a timer event handler is
-   provided in MainWindow::timer_event() (which also calls
-   EfaxController::timer_event()) which deals with interrupts and
-   child exit handling.
+6. this program used libsigc++ prior to version 3.2.0.
+   sigc::trackable in libsigc++ is not thread safe - amongst other
+   things, a class inheriting from sigc::trackable will, via that
+   inheritance, have a std::list object keeping track of slots
+   connected to any of its non-static methods.  Each sigc::slot object
+   also, via sigc::slot_rep, keeps its own sigc::trackable object to
+   track any sigc::connection objects which it needs to inform about
+   its demise.  sigc::signal objects also keep lists of slots, which
+   will be updated with a call to its connect() and disconnect()
+   methods.  This required a few rules to be observed in
+   multi-threaded applications:
+
+     - Unless special additional synchronisation is employed, regard
+       any particular sigc::signal object as "owned" by the thread
+       which created it.  Only that thread should connect or
+       disconnect slots with respect to the signal object, and only
+       that thread should emit() or call operator()() on it.
+
+     - Unless special additional synchronisaton is employed, regard
+       any sigc::connection object as owned by the thread which
+       created the slot and called the method which provided the
+       sigc::connection object.  Only that thread should call
+       sigc::connection methods on the object.
+
+     - Unless you really know what you are doing, do not copy slots
+       between threads.  In addition never ever copy to another thread
+       a slot which references a non-static method of a class derived
+       from sigc::trackable.
+
+     - If a particular class object derives from sigc::trackable, only
+       one thread should create slots representing any of its
+       non-static methods (ie create slots with sigc::mem_fun()).
+       Regard the first thread to create such a slot as owning the
+       relevant object for the purpose of creating further slots
+       referencing any of its methods.  (It is safe however to make
+       objects of the opaque callback classes in the
+       src/utils/callback.h file in any thread, as these are thread
+       safe and do not have trackability.)
+
+   From version 3.2.0, this program now uses the thread-safe Releaser
+   and EmitterArg<> classes in the c++-gtk-utils library.
+
+
 
 *********************************************************************/
 
@@ -225,6 +247,7 @@ struct Prog_config {
   bool fax_received_popup;
   bool fax_received_exec;
   bool addr_in_header;
+  bool redial;
   std::string my_name;
   std::string my_number;
   std::string page_size;  // either 'a4' 'letter' or 'legal'
@@ -237,6 +260,7 @@ struct Prog_config {
   std::string logfile_name;
   std::string sock_server_port;
   std::string fax_received_prog;
+  std::string redial_interval;
   std::vector<std::string> parms;
   std::vector<std::string> permitted_clients_list;
   pid_t efax_controller_child_pid;
