@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2006 Chris Vine
+/* Copyright (C) 2001 to 2010 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -14,17 +14,17 @@ COPYING distributed with the source files.
 #include <unistd.h>
 
 #include <string>
+#include <utility>
 
-#include <gtk/gtkwidget.h>
-#include <gtk/gtkwindow.h>
-
-#include <sigc++/sigc++.h>
+#include <gtk/gtk.h>
 
 #include "settings_help.h"
-#include "utils/window.h"
-#include "utils/widget.h"
-#include "utils/gobj_handle.h"
-#include "utils/utf8_utils.h"
+
+#include <c++-gtk-utils/gobj_handle.h>
+#include <c++-gtk-utils/window.h>
+#include <c++-gtk-utils/widget.h>
+#include <c++-gtk-utils/emitter.h>
+#include <c++-gtk-utils/convert.h>
 
 
 namespace { // we put the function in anonymous namespace in settings.cpp
@@ -44,7 +44,8 @@ class IdentityPage: public MainWidgetBase {
 public:
   friend void IdentityPageCB::identity_page_button_clicked(GtkWidget*, void*);
 
-  sigc::signal2<void, const std::string&, const std::string&> show_help_sig;
+  //sigc::signal2<void, const std::string&, const std::string&> show_help_sig;
+  EmitterArg<const std::pair<std::string, std::string>&> show_help_sig;
   std::string get_name(void) const;
   std::string get_number(void) const;
   void set_name(const std::string& name);
@@ -83,7 +84,8 @@ class ModemPage: public MainWidgetBase {
 public:
   friend void ModemPageCB::modem_page_button_clicked(GtkWidget*, void*);
 
-  sigc::signal2<void, const std::string&, const std::string&> show_help_sig;
+  //sigc::signal2<void, const std::string&, const std::string&> show_help_sig;
+  EmitterArg<const std::pair<std::string, std::string>&> show_help_sig;
   std::string get_device(void) const;
   std::string get_lock(void) const;
   std::string get_capabilities(void) const;
@@ -120,7 +122,8 @@ class ParmsPage: public MainWidgetBase {
 public:
   friend void ParmsPageCB::parms_page_button_clicked(GtkWidget*, void*);
 
-  sigc::signal2<void, const std::string&, const std::string&> show_help_sig;
+  //sigc::signal2<void, const std::string&, const std::string&> show_help_sig;
+  EmitterArg<const std::pair<std::string, std::string>&> show_help_sig;
   std::string get_init(void) const;
   std::string get_reset(void) const;
   std::string get_parms(void) const;
@@ -145,6 +148,7 @@ class PrintPage: public MainWidgetBase {
   GtkWidget* popup_label_p;
   GobjHandle<GtkWidget> gtkprint_check_button_h;
   GtkWidget* command_entry_p;
+  GtkWidget* shrink_label_p;
   GtkWidget* shrink_spin_button_p;
   GtkWidget* popup_check_button_p;
   GobjHandle<GtkWidget> gtkprint_help_button_h;
@@ -156,7 +160,8 @@ class PrintPage: public MainWidgetBase {
 public:
   friend void PrintPageCB::print_page_button_clicked(GtkWidget*, void*);
 
-  sigc::signal2<void, const std::string&, const std::string&> show_help_sig;
+  //sigc::signal2<void, const std::string&, const std::string&> show_help_sig;
+  EmitterArg<const std::pair<std::string, std::string>&> show_help_sig;
   std::string get_gtkprint(void) const;
   std::string get_command(void) const;
   std::string get_shrink(void) const;
@@ -184,7 +189,8 @@ class ViewPage: public MainWidgetBase {
 public:
   friend void ViewPageCB::view_page_button_clicked(GtkWidget*, void*);
 
-  sigc::signal2<void, const std::string&, const std::string&> show_help_sig;
+  //sigc::signal2<void, const std::string&, const std::string&> show_help_sig;
+  EmitterArg<const std::pair<std::string, std::string>&> show_help_sig;
   std::string get_ps_view_command(void) const;
   void set_ps_view_command(const std::string& command);
   void clear(void);
@@ -216,7 +222,8 @@ class SockPage: public MainWidgetBase {
 public:
   friend void SockPageCB::sock_page_button_clicked(GtkWidget*, void*);
 
-  sigc::signal2<void, const std::string&, const std::string&> show_help_sig;
+  //sigc::signal2<void, const std::string&, const std::string&> show_help_sig;
+  EmitterArg<const std::pair<std::string, std::string>&> show_help_sig;
   std::string get_run_server(void) const ;
   std::string get_popup(void) const;
   std::string get_port(void) const;
@@ -252,7 +259,8 @@ class ReceivePage: public MainWidgetBase {
 public:
   friend void ReceivePageCB::receive_page_button_clicked(GtkWidget*, void*);
 
-  sigc::signal2<void, const std::string&, const std::string&> show_help_sig;
+  //sigc::signal2<void, const std::string&, const std::string&> show_help_sig;
+  EmitterArg<const std::pair<std::string, std::string>&> show_help_sig;
   std::string get_popup(void) const;
   std::string get_exec(void) const;
   std::string get_prog(void) const;
@@ -276,19 +284,30 @@ class SendPage: public MainWidgetBase {
   GtkWidget* standard_button_p;
   GtkWidget* fine_button_p;
   GtkWidget* header_check_button_p;
+  GtkWidget* redial_check_button_p;
+  GtkWidget* redial_spin_label_p;
+  GtkWidget* redial_spin_button_p;
   GtkWidget* dial_prefix_entry_p;
   GtkWidget* res_help_button_p;
   GtkWidget* header_help_button_p;
+  GtkWidget* redial_help_button_p;
   GtkWidget* dial_prefix_help_button_p;
+
+  void redial_check_button_toggled_impl(void);
 public:
   friend void SendPageCB::send_page_button_clicked(GtkWidget*, void*);
 
-  sigc::signal2<void, const std::string&, const std::string&> show_help_sig;
+  //sigc::signal2<void, const std::string&, const std::string&> show_help_sig;
+  EmitterArg<const std::pair<std::string, std::string>&> show_help_sig;
   std::string get_res(void) const;
   std::string get_addr_in_header(void) const;
+  std::string get_redial(void) const;
+  std::string get_redial_interval(void) const;
   std::string get_dial_prefix(void) const;
   void set_res(const std::string& res);
   void set_addr_in_header(const std::string& res);
+  void set_redial(const std::string& redial);
+  void set_redial_interval(const std::string& interval);
   void set_dial_prefix(const std::string& prefix);
   void clear(void);
   SendPage(const int);
@@ -308,7 +327,8 @@ class LoggingPage: public MainWidgetBase {
 public:
   friend void LoggingPageCB::logging_page_button_clicked(GtkWidget*, void*);
 
-  sigc::signal2<void, const std::string&, const std::string&> show_help_sig;
+  //sigc::signal2<void, const std::string&, const std::string&> show_help_sig;
+  EmitterArg<const std::pair<std::string, std::string>&> show_help_sig;
   std::string get_logfile(void) const;
   void set_logfile(const std::string& logfile);
   void clear(void);
@@ -332,7 +352,8 @@ class PagePage: public MainWidgetBase {
 public:
   friend void PagePageCB::page_page_button_clicked(GtkWidget*, void*);
 
-  sigc::signal2<void, const std::string&, const std::string&> show_help_sig;
+  //sigc::signal2<void, const std::string&, const std::string&> show_help_sig;
+  EmitterArg<const std::pair<std::string, std::string>&> show_help_sig;
   std::string get_page(void) const;
   void set_page(const std::string& page);
   void clear(void);
@@ -346,7 +367,9 @@ namespace SettingsDialogCB {
 }
 }
 
-class SettingsDialog: public sigc::trackable, public WinBase {
+class SettingsDialog: public WinBase {
+
+  static int dialog_count;
 
   const int standard_size;
   bool is_home_config;
@@ -376,19 +399,21 @@ class SettingsDialog: public sigc::trackable, public WinBase {
   bool get_rcfile_path(bool search_localfile = true);
   void get_reset_settings_prompt(void);
   void get_reset_settings(void);
-  void show_help(const std::string& message, const std::string& caption);
+  void show_help(const std::pair<std::string, std::string>&);
   void strip(std::string&);
   bool is_ascii(const std::string&);
 public:
   friend void SettingsDialogCB::settings_dialog_button_clicked(GtkWidget*, void*);
 
+  static bool is_dialog(void) {return dialog_count;}
+
   // this signal can be used to indicate that settings have changed
   // it is emitted after this dialog has rewritten the ~/.efax-gtkrc
   // and has called configure_prog() and passes the messages
-  // returned by configure_prog() together with a bool indicating
-  // whether the socket server should be running
-  sigc::signal1<void, const std::string&> accepted;
+  // returned by configure_prog()
+  EmitterArg<const std::string&> accepted;
   SettingsDialog(const int standard_size, GtkWindow* parent_p, bool skip_old_settings = false);
+  ~SettingsDialog(void) {--dialog_count;}
 };
 
 #endif
