@@ -1,4 +1,4 @@
-/* Copyright (C) 2001 to 2006 Chris Vine
+/* Copyright (C) 2001 to 2006 and 2009 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -13,45 +13,20 @@ COPYING distributed with the source files.
 #include <cstdlib>
 #include <cstring>
 #include <vector>
+#include <algorithm>
+#include <ios>
+#include <ostream>
+#include <sstream>
 
-#include <gtk/gtkcontainer.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkbox.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkbbox.h>
-#include <gtk/gtkhbbox.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtktoolbar.h>
-#include <gtk/gtktooltips.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtkliststore.h>
-#include <gtk/gtkcellrenderer.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtktreeviewcolumn.h>
-#include <gtk/gtkversion.h>
-
-#include <gdk/gdkpixbuf.h>
+#include <gdk/gdk.h>
 
 #include "socket_list.h"
 #include "dialogs.h"
 #include "socket_list_icons.h"
-#include "utils/utf8_utils.h"
 #include "utils/toolbar_append_widget.h"
 
-#ifdef HAVE_OSTREAM
-#include <ios>
-#include <ostream>
-#endif
-
-#ifdef HAVE_STRINGSTREAM
-#include <sstream>
-#else
-#include <strstream>
-#endif
+#include <c++-gtk-utils/mem_fun.h>
+#include <c++-gtk-utils/convert.h>
 
 #ifdef ENABLE_NLS
 #include <libintl.h>
@@ -78,11 +53,11 @@ void SocketListDialogCB::socket_list_button_clicked(GtkWidget* widget_p, void* d
     instance_p->remove_file_prompt();
   }
   else if (widget_p == instance_p->close_button_p) {
-    instance_p->close_slot();
+    instance_p->close();
   }
   else {
     write_error("Callback error in SocketListDialogCB::socket_list_button_clicked()\n");
-    instance_p->close_slot();
+    instance_p->close();
   }
 }
 
@@ -110,6 +85,22 @@ void SocketListDialogCB::socket_list_set_buttons(GtkTreeSelection*, void* data)
   }
 }
 
+gboolean SocketListDialogCB::tree_view_mouse_click(GtkWidget*,
+						   GdkEventButton* event_p, void* data) {
+  
+  if (event_p->type == GDK_2BUTTON_PRESS) {
+    SocketListDialog* instance_p = static_cast<SocketListDialog*>(data);
+    int x, y;
+    gtk_widget_get_pointer(GTK_WIDGET(instance_p->tree_view_p), &x, &y);
+    if (x >= 0 && y >= 0
+	&& gtk_tree_view_get_dest_row_at_pos(instance_p->tree_view_p,
+					     x, y, 0, 0)) {
+      instance_p->view_file();
+    }
+  }
+  return false;
+}
+
 } // anonymous namespace
 
 
@@ -170,13 +161,12 @@ SocketListDialog::SocketListDialog(std::pair<SharedPtr<FilenamesList>,
   gtk_tree_selection_set_mode(selection_p, GTK_SELECTION_SINGLE);
 
   // set up the tool bar
+#if GTK_CHECK_VERSION(2,16,0)
+  gtk_orientable_set_orientation(GTK_ORIENTABLE(toolbar_p), GTK_ORIENTATION_HORIZONTAL);
+#else
   gtk_toolbar_set_orientation(toolbar_p, GTK_ORIENTATION_HORIZONTAL);
+#endif
   gtk_toolbar_set_style(toolbar_p, GTK_TOOLBAR_ICONS);
-  gtk_toolbar_set_tooltips(toolbar_p, true);
-
-  // this is OK as tooltips is a public member of GtkToolbar
-  // is this necessary after calling gtk_toolbar_set_tooltips()?
-  gtk_tooltips_enable(toolbar_p->tooltips);
 
   GtkWidget* image_p;
   { // scope block for the GobjHandle
@@ -186,7 +176,7 @@ SocketListDialog::SocketListDialog(std::pair<SharedPtr<FilenamesList>,
     gtk_container_add(GTK_CONTAINER(view_button_p), image_p);
   }
   toolbar_append_widget(toolbar_p, view_button_p,
-			gettext("View selected file"), "");
+			gettext("View selected file"));
   gtk_widget_set_sensitive(view_button_p, false);
 
   { // scope block for the GobjHandle
@@ -196,7 +186,7 @@ SocketListDialog::SocketListDialog(std::pair<SharedPtr<FilenamesList>,
     gtk_container_add(GTK_CONTAINER(remove_button_p), image_p);
   }
   toolbar_append_widget(toolbar_p, remove_button_p,
-			gettext("Remove selected file from list"), "");
+			gettext("Remove selected file from list"));
   gtk_widget_set_sensitive(remove_button_p, false);
 
   GtkWidget* send_hbox_p = gtk_hbox_new(false, 0);
@@ -210,7 +200,7 @@ SocketListDialog::SocketListDialog(std::pair<SharedPtr<FilenamesList>,
     gtk_container_add(GTK_CONTAINER(send_button_p), send_hbox_p);
   }
   toolbar_append_widget(toolbar_p, send_button_p,
-			gettext("Choose the selected fax for sending"), "");
+			gettext("Choose the selected fax for sending"));
   gtk_widget_set_sensitive(send_button_p, false);
 
   gtk_table_attach(table_p, GTK_WIDGET(socket_list_scrolled_window_p),
@@ -240,18 +230,27 @@ SocketListDialog::SocketListDialog(std::pair<SharedPtr<FilenamesList>,
   g_signal_connect(G_OBJECT(selection_p), "changed",
 		   G_CALLBACK(SocketListDialogCB::socket_list_set_buttons), this);
 
+  // capture a double click on the tree view for viewing a fax for sending
+  gtk_widget_add_events(GTK_WIDGET(tree_view_p), GDK_BUTTON_PRESS_MASK);
+  g_signal_connect(G_OBJECT(tree_view_p), "button_press_event",
+		   G_CALLBACK(SocketListDialogCB::tree_view_mouse_click), this);
+
+#if GTK_CHECK_VERSION(2,20,0)
+  gtk_widget_set_can_default(close_button_p, true);
+#else
   GTK_WIDGET_SET_FLAGS(close_button_p, GTK_CAN_DEFAULT);
+#endif
 
   gtk_container_set_border_width(GTK_CONTAINER(table_p), standard_size/3);
   gtk_container_add(GTK_CONTAINER(get_win()), socket_list_box_p);
   //gtk_window_set_position(get_win(), GTK_WIN_POS_CENTER);
   gtk_window_set_default_size(get_win(), standard_size * 15, standard_size * 12);
-#if GTK_CHECK_VERSION(2,4,0)
+
   gtk_toolbar_set_show_arrow(GTK_TOOLBAR(toolbar_p), false);
-#endif
+
   gtk_widget_show_all(GTK_WIDGET(get_win()));
 
-  // not needed if gtk_toolbar_set_show_arrow() has been set false, or GTK+ version < 2.4
+  // not needed if gtk_toolbar_set_show_arrow() has been set false
   /*
   gtk_widget_set_size_request(GTK_WIDGET(toolbar_p), 
 			      send_button_p->allocation.width + view_button_p->allocation.width
@@ -265,10 +264,6 @@ SocketListDialog::~SocketListDialog(void) {
   is_socket_list--;
 }
 
-void SocketListDialog::on_delete_event(void) {
-  close_slot();
-}
-
 void SocketListDialog::send_fax_impl(void) {
 
   std::pair<std::string, std::string> fax_to_send;
@@ -296,50 +291,44 @@ void SocketListDialog::send_fax_impl(void) {
   }
 }
 
-void SocketListDialog::close_slot(void) {
-  dialog_closed();
+void SocketListDialog::close_cb(void) {
   close();
 }
 
 void SocketListDialog::set_socket_list_rows(std::pair<SharedPtr<FilenamesList>,
 					              SharedPtr<Thread::Mutex::Lock> > filenames_pair) {
 
-  FilenamesList::const_iterator filenames_iter;
-
   gtk_list_store_clear(GTK_LIST_STORE(list_store_h.get()));
 
-  for (filenames_iter = filenames_pair.first->begin();
-       filenames_iter != filenames_pair.first->end();
-       ++filenames_iter) {
+  // filenames_pair.first references a std::list object containing
+  // another pair as value type, the first member of which is a
+  // temporary file name created by mkstemp() in
+  // SocketServer::accept_client(), and the second member of which
+  // contains the print job number assigned by
+  // SocketServer::add_file()
 
-    // filenames_iter->first will contain the temporary file name created by mkstemp
-    // in void SocketServer::accept_client()
-    // filenames_iter->second will contain the print job number assigned by
-    // SocketServer::add_file()
+  // filenames_pair.second is just a tunnelling lock, which gives us
+  // exclusive access to the FilenamesList object maintained by the
+  // SocketServer object while we are iterating through it in this
+  // function
 
-    // get a row to insert the file name in
-    GtkTreeIter row_iter;
-    gtk_list_store_append(GTK_LIST_STORE(list_store_h.get()), &row_iter);
+  std::for_each(filenames_pair.first->begin(), filenames_pair.first->end(),
+		MemFun::make(*this, &SocketListDialog::set_socket_list_item));
+}
 
-    // insert fax label and fax filename
-#ifdef HAVE_STRINGSTREAM
-    std::ostringstream strm;
-    strm << gettext("PRINT JOB: ") << filenames_iter->second;
-    gtk_list_store_set(GTK_LIST_STORE(list_store_h.get()), &row_iter,
-		       ModelColumns::fax_label, strm.str().c_str(),
-		       ModelColumns::fax_filename, filenames_iter->first.c_str(),
-		       -1);
-#else
-    std::ostrstream strm;
-    strm << gettext("PRINT JOB: ") << filenames_iter->second << std::ends;
-    const char* fax_label_p = strm.str();
-    gtk_list_store_set(GTK_LIST_STORE(list_store_h.get()), &row_iter,
-		       ModelColumns::fax_label, (const gchar*)fax_label_p,
-		       ModelColumns::fax_filename, filenames_iter->first.c_str(),
-		       -1);
-    delete[] fax_label_p;
-#endif
-  }
+void SocketListDialog::set_socket_list_item(const std::pair<std::string, unsigned int>& item) {
+
+  // get a row to insert the file name in
+  GtkTreeIter row_iter;
+  gtk_list_store_append(GTK_LIST_STORE(list_store_h.get()), &row_iter);
+
+  // insert fax label and fax filename
+  std::ostringstream strm;
+  strm << gettext("PRINT JOB: ") << item.second;
+  gtk_list_store_set(GTK_LIST_STORE(list_store_h.get()), &row_iter,
+		     ModelColumns::fax_label, strm.str().c_str(),
+		     ModelColumns::fax_filename, item.first.c_str(),
+		     -1);
 }
 
 std::pair<const char*, char* const*> SocketListDialog::get_view_file_parms(const std::string& file_name) {
@@ -498,7 +487,7 @@ void SocketListDialog::remove_file_prompt(void) {
 
       PromptDialog* dialog_p = new PromptDialog(msg.c_str(), gettext("efax-gtk: Remove queued fax"),
 						standard_size, get_win());
-      dialog_p->accepted.connect(sigc::mem_fun(*this, &SocketListDialog::remove_file));
+      dialog_p->accepted.connect(Callback::make(*this, &SocketListDialog::remove_file));
       // there is no memory leak -- the memory will be deleted when PromptDialog closes
     }
   }
@@ -525,7 +514,7 @@ void SocketListDialog::remove_file(void) {
     // which will call SocketServer::remove_file() which will then
     // invoke the SocketServer::filelist_changed_notify Notifier
     // object which will in turn invoke the
-    // MainWindow::socket_filelist_changed_notify_slot() which will
+    // MainWindow::socket_filelist_changed_notify_cb() which will
     // then call SocketListDialog::set_socket_list_rows() and so
     // refresh the socket file list.  The server object will also
     // clean up by deleting the temporary file it created.  We do not
