@@ -1,4 +1,4 @@
-/* Copyright (C) 2004 and 2006 Chris Vine
+/* Copyright (C) 2004 and 2006 and 2009 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -6,29 +6,13 @@ COPYING distributed with the source files.
 
 */
 
-#include <gtk/gtklabel.h>
-#include <gtk/gtkbutton.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtkbox.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkbbox.h>
-#include <gtk/gtkhbbox.h>
+#include <ios>
+#include <ostream>
+#include <sstream>
 
 #include "addressbook.h"
 #include "socket_notify.h"
 
-#ifdef HAVE_OSTREAM
-#include <ios>
-#include <ostream>
-#endif
-
-#ifdef HAVE_STRINGSTREAM
-#include <sstream>
-#else
-#include <strstream>
-#endif
-
 #ifdef ENABLE_NLS
 #include <libintl.h>
 #endif
@@ -42,8 +26,8 @@ void SocketNotifyDialogCB::socket_notify_button_clicked(GtkWidget* widget_p, voi
     if (!AddressBook::get_is_address_list()) {
       AddressBook* dialog_p = new AddressBook(instance_p->standard_size,
 					      instance_p->get_win());
-      dialog_p->accepted.connect(sigc::mem_fun(*instance_p,
-					       &SocketNotifyDialog::set_number_slot));
+      dialog_p->accepted.connect(Callback::make(*instance_p,
+						&SocketNotifyDialog::set_number_cb));
       // there is no memory leak -- AddressBook will delete its own memory
       // when it is closed
     }
@@ -54,10 +38,14 @@ void SocketNotifyDialogCB::socket_notify_button_clicked(GtkWidget* widget_p, voi
     instance_p->close();
   }
   else if (widget_p == instance_p->queue_button_p) {
+    gtk_widget_hide_all(GTK_WIDGET(instance_p->get_win()));
+    instance_p->queue_sig();
     instance_p->close();
   }
   else {
     write_error("Callback error in SocketNotifyDialogCB::socket_notify_button_clicked()\n");
+    gtk_widget_hide_all(GTK_WIDGET(instance_p->get_win()));
+    instance_p->queue_sig();
     instance_p->close();
   }
 }
@@ -81,37 +69,18 @@ SocketNotifyDialog::SocketNotifyDialog(const int size,
   GtkWidget* window_box_p = gtk_vbox_new(false, 0);
 
   gtk_button_box_set_layout(GTK_BUTTON_BOX(button_box_p), GTK_BUTTONBOX_END);
-  gtk_button_box_set_spacing(GTK_BUTTON_BOX(button_box_p), standard_size/3);
+  gtk_box_set_spacing(GTK_BOX(button_box_p), standard_size/3);
   gtk_container_add(GTK_CONTAINER(button_box_p), queue_button_p);
   gtk_container_add(GTK_CONTAINER(button_box_p), send_button_p);
 
   fax_name.second = fax_pair.first;
-#ifdef HAVE_STRINGSTREAM
+
   std::ostringstream strm;
   strm << gettext("PRINT JOB: ") << fax_pair.second;
-
   fax_name.first = strm.str();
   strm << gettext(" has been received on socket.\n"
 		  "Send or queue fax?");
   GtkWidget* label_p = gtk_label_new(strm.str().c_str());
-#else
-  std::ostrstream strm1;
-  std::ostrstream strm2;
-
-  strm1 << gettext("PRINT JOB: ") << fax_pair.second << std::ends;
-  const char* text_p = strm1.str();
-  fax_name.first = text_p;
-  delete[] text_p;
-
-  strm2 << gettext("PRINT JOB: ") << fax_pair.second
-	<< gettext(" has been received on socket.\n"
-		   "Send or queue fax?")
-	<< std::ends;
-
-  text_p = strm2.str();
-  GtkWidget* label_p = gtk_label_new(text_p);
-  delete[] text_p;
-#endif
 
   gtk_widget_set_size_request(number_entry_p, standard_size * 7, standard_size);
   
@@ -129,9 +98,15 @@ SocketNotifyDialog::SocketNotifyDialog(const int size,
   g_signal_connect(G_OBJECT(queue_button_p), "clicked",
 		   G_CALLBACK(SocketNotifyDialogCB::socket_notify_button_clicked), this);
 
+#if GTK_CHECK_VERSION(2,20,0)
+  gtk_widget_set_can_default(number_button_p, true);
+  gtk_widget_set_can_default(send_button_p, true);
+  gtk_widget_set_can_default(queue_button_p, true);
+#else
   GTK_WIDGET_SET_FLAGS(number_button_p, GTK_CAN_DEFAULT);
   GTK_WIDGET_SET_FLAGS(send_button_p, GTK_CAN_DEFAULT);
   GTK_WIDGET_SET_FLAGS(queue_button_p, GTK_CAN_DEFAULT);
+#endif
 
   gtk_container_add(GTK_CONTAINER(get_win()), window_box_p);
   gtk_container_set_border_width(GTK_CONTAINER(get_win()), standard_size/2);
@@ -140,7 +115,7 @@ SocketNotifyDialog::SocketNotifyDialog(const int size,
   gtk_widget_show_all(GTK_WIDGET(get_win()));
 }
 
-void SocketNotifyDialog::set_number_slot(const std::string& number) {
+void SocketNotifyDialog::set_number_cb(const std::string& number) {
 
   if (!number.empty()) {
     gtk_entry_set_text(GTK_ENTRY(number_entry_p), number.c_str());
@@ -156,3 +131,9 @@ void SocketNotifyDialog::send_signals(void) {
   fax_number_sig(gtk_entry_get_text(GTK_ENTRY(number_entry_p)));
   sendfax_sig();
 }
+
+void SocketNotifyDialog::on_delete_event(void) {
+  gtk_widget_hide_all(GTK_WIDGET(get_win()));
+  queue_sig();
+  close();
+}
