@@ -1,4 +1,4 @@
-/* Copyright (C) 2004 and 2005 Chris Vine
+/* Copyright (C) 2004 and 2005 and 2009 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -14,12 +14,10 @@ COPYING distributed with the source files.
 #include <string>
 #include <utility>
 
-#include <gtk/gtkwidget.h>
-#include <gtk/gtkwindow.h>
+#include <gtk/gtk.h>
 
-#include <sigc++/sigc++.h>
-
-#include "utils/window.h"
+#include <c++-gtk-utils/window.h>
+#include <c++-gtk-utils/emitter.h>
 
 
 namespace { // we put the function in anonymous namespace in socket_notify.cpp
@@ -29,7 +27,7 @@ namespace SocketNotifyDialogCB {
 }
 }
 
-class SocketNotifyDialog: public sigc::trackable, public WinBase {
+class SocketNotifyDialog: public WinBase {
   const int standard_size;
   std::pair<std::string, std::string> fax_name;
 
@@ -39,14 +37,17 @@ class SocketNotifyDialog: public sigc::trackable, public WinBase {
 
   GtkWidget* number_entry_p;
 
-  void set_number_slot(const std::string&);
+  void set_number_cb(const std::string&);
   void send_signals(void);
+protected:
+  virtual void on_delete_event(void);
 public:
   friend void SocketNotifyDialogCB::socket_notify_button_clicked(GtkWidget*, void*);
 
-  sigc::signal1<void, const std::pair<std::string, std::string>&> fax_name_sig;
-  sigc::signal1<void, const std::string&> fax_number_sig;
-  sigc::signal0<void> sendfax_sig;
+  EmitterArg<const std::pair<std::string, std::string>&> fax_name_sig;
+  EmitterArg<const std::string&> fax_number_sig;
+  Emitter sendfax_sig;
+  Emitter queue_sig;
 
   SocketNotifyDialog(const int standard_size, const std::pair<std::string, unsigned int>&);
 };
