@@ -1,4 +1,4 @@
-/* Copyright (C) 2003 to 2006 Chris Vine
+/* Copyright (C) 2003 to 2006 and 2009 Chris Vine
 
 This program is distributed under the General Public Licence, version 2.
 For particulars of this and relevant disclaimers see the file
@@ -10,6 +10,8 @@ COPYING distributed with the source files.
 #include <fstream>
 #include <vector>
 #include <algorithm>
+#include <ios>
+#include <ostream>
 #include <cstring>
 
 #include <sys/stat.h>
@@ -24,13 +26,9 @@ COPYING distributed with the source files.
 #include <pthread.h>
 
 #include "socket_server.h"
-#include "utils/shared_handle.h"
-#include "utils/callback.h"
 
-#ifdef HAVE_OSTREAM
-#include <ios>
-#include <ostream>
-#endif
+#include <c++-gtk-utils/shared_handle.h>
+#include <c++-gtk-utils/callback.h>
 
 #ifdef HAVE_STREAM_IMBUE
 #include <locale>
@@ -60,8 +58,8 @@ SocketServer::SocketServer(void): server_running(false),
 
   fax_to_send.second = 0;
 
-  stdout_notify.connect(sigc::mem_fun(*this, &SocketServer::write_stdout_dispatcher_slot));
-  socket_error_notify.connect(sigc::mem_fun(*this, &SocketServer::cleanup));
+  stdout_notify.connect(Callback::make(*this, &SocketServer::write_stdout_dispatcher_cb));
+  socket_error_notify.connect(Callback::make(*this, &SocketServer::cleanup));
 
   // make sure that we have the $HOME/efax-gtk-server directory
   // (we do not need to lock working_dir_mutex as the socket server thread cannot
@@ -467,7 +465,7 @@ void SocketServer::write_stdout_from_thread(const char* message) {
   stdout_notify();
 }
 
-void SocketServer::write_stdout_dispatcher_slot(void) {
+void SocketServer::write_stdout_dispatcher_cb(void) {
 
   Thread::Mutex::Lock lock(stdout_mutex);
 
