@@ -22,26 +22,15 @@ License as follows:
 
 */
 
-#include <gtk/gtkversion.h>
-#if GTK_CHECK_VERSION(2,4,0)
-#include <gtk/gtktoolitem.h>
-#include <gtk/gtktooltips.h>
-#include <gtk/gtkcontainer.h>
-#endif
-
 #include "toolbar_append_widget.h"
 
 
 void toolbar_append_widget(GtkToolbar* toolbar_p, GtkWidget* widget_p,
-			   const char* tooltip_text_p, const char* tooltip_private_text_p) {
+			   const char* tooltip_text_p) {
 
-#if GTK_CHECK_VERSION(2,4,0)
   GtkToolItem* toolitem_p = gtk_tool_item_new();
   gtk_container_add(GTK_CONTAINER(toolitem_p), widget_p);
   // this is OK as tooltips is a public member of GtkToolbar
-  gtk_tool_item_set_tooltip(toolitem_p, toolbar_p->tooltips, tooltip_text_p, tooltip_private_text_p);
+  gtk_tool_item_set_tooltip_text(toolitem_p, tooltip_text_p);
   gtk_toolbar_insert(toolbar_p, toolitem_p, -1);
-#else
-  gtk_toolbar_append_widget(toolbar_p, widget_p, tooltip_text_p, tooltip_private_text_p);
-#endif
 }
